<?
include( '../include/config.inc.php' );
?>
<h3>Overview</h3>
<p>
This is an open source equilibrium climate model system for testing the effects of greenhouse gasses in a simplified atmosphere.
</p>
<p>
The model uses ten layers of atmosphere of equal pressure change (100hPa), and calculates the energy fluxes between the layers.
The longwave fluxes are determined by the greenhouse gas composition of the layers. Each layer absorbs radiation and emits it according to spectral profile of the greenhouse gasses within the layer.
</p>
<p>
The greenhouse gasses CO2, CH4, N20 have constant concentrations with each layer, the Water Vapour levels drop off with height. Ozone has a higher concentration in the upper level only. 
You can set the scenario for each 'dry' gas, and this will multiple its concentration by this amount. For Ozone, the multiplication will be applied to the concentration of each layer.
</p>
<p>
To set vapour levels, determine a relative humidity for each layer. The vapour pressure of each layer will be calculated accordingly based on the initial temperature profile. The initial temperature profile 
is the default result with the atmosphere configuration left unchanged.
</p>
<p>
The H20 vapour pressures will be recalculated with each correction process if "Adjust vapour levels with deduced temperatures" is left checked. The model will continue correcting results until the 
deviations (Root Mean Square Differences) from previous corrections become less than <?=TOLERANCE?> &deg;
</p>
<p>
Clouds can be set for one level only, reflections of both incoming solar radiation and outgoing longwave radiation are both calculated. You can determine the cloud coverage and its reflective properties.
Ground reflections of longwave radiation are calculated also, so too are multiple reflections from cloud to ground.
</p>
<p>
Solar radiation input is only affected by cloud, some allowance for ozone absorption at high levels and H2O absorption in the lower atmosphere is made, but the method is very crude.
Changing Ozone levels in the atmosphere set-up will not affect the solar input, only the infrared absorption. Hence this model is not at the moment designed for testing changes in Ozone concentrations.
Further work on the solar input function is required. So too is the aerosol properties of H2O left as fixed in the model, with no regard for how changes may affect solar input.
For this reason, the model could be overestimating the vapour feedback effects by not taking into account the negative feedback effect of H2O aerosol behaviour.
</p>
<p>
<embed src="include/process.php?actiontype=getLayerDiagram" width="800" height="700"
type="image/svg+xml"/>
</p>
<p>
Above diagram shows the 11 Flux surfaces FL(0) - FL(10) where flux surface calculations are made. Denoted on the right are the 10 layers of atmosphere of 100hPa thickness and surface for which 
temperatures are assigned.<br/>
The lines depict how energy from layer 3 reach flux surface 7. The direct path, attenuated by the absorptions of layers 4,5,6 and 7 only. The reflected path which is attenuated by the 2 and 1 twice plus the surface albedo
as well as the layers above.<br/>
From then, we have what is reflected by clouds and surface and what may pass through clouds.<br/>
The energy paths depict one data point in a matrix of 11X11 datapoints representing the flux of 11 different layers (including surface) reaching 11 different flux layers.<br/>
<br/>
At each  flux surface the total flux upwards must balance the flux downwards, of which the latter includes the Solar input.
 

</p>
<h3>Convection</h3>
<p>
Convection is the weakest aspect of this model. Convection is a product of a dynamic system, so an equilibrium model will only be able to approximate this.
Nevertheless, by assuming that convection occurs when the Temperature Gradient with height exceeds the Adiabatic Lapse rate, and making this proportional to the square of the difference, we get a 
fairly good depiction of convection.<br/>
Convection can be thought of as two types: dry and wet. The former being the convection that occurs when the temperature gradient exceed the dry adiabatic lapse rate 
(the change in temperature air will naturally experience if lifted and depressurised). The latter being the temperature change when the air is saturated with water vapour.
In nature, wet convection occurs either if the air is 100% saturated, or if air below is being lifted through dry convection and cooling until it is saturated.
The complexity of wet-convection make this even more spurious to model, however, by applying the same methodology, the model makes a good approximation of wet-convection. It may be thought that convection 
modelling is essentially a probability model. The equilibrium model may be modelling an average condition over wide spaces and time (night and day), with the assumption that somewhere convection is occurring 
affecting the overall temperature profile.
</p>

<p style="font-style: italic">
 Dry Convection = C<sub>d</sub>( <sup>&Delta;T</sup>/<sub>&Delta;P</sub> - L<sub>d</sub> / <sub>&#254; g</sub> )<sup>2</sup>  &nbsp; Providing: ( <sup>&Delta;T</sup>/<sub>&Delta;P</sub> - L<sub>d</sub> / <sub>&#254; g</sub> ) < 0<br/>
 Wet Convection = C<sub>w</sub>( <sup>&Delta;T</sup>/<sub>&Delta;P</sub> - L<sub>w</sub> / <sub>&#254; g</sub> )<sup>2</sup>  &nbsp; Providing: ( <sup>&Delta;T</sup>/<sub>&Delta;P</sub> - L<sub>w</sub> / <sub>&#254; g</sub> ) < 0<br/>
 C<sub>d</sub> - Coefficient of Dry Convection (Wm<sup>-2</sup> K&deg;<sup>-2</sup>hPa<sup>2</sup>)<br/>
 C<sub>w</sub> - Coefficient of Wet Convection  (Wm<sup>-2</sup> K&deg;<sup>-2</sup>hPa<sup>2</sup>)<br/>
 L<sub>d</sub> - Dry Adiabatic Lapse rate (K&deg;m<sup>-1</sup>)<br/>
 L<sub>w</sub> - Wet Adiabatic Lapse rate (K&deg;m<sup>-1</sup>)<br/>
 &#254; - Air Density (Kg m<sup>-3</sup>)<br/>
 g - Acceleration from gravity (ms<sup>-2)</sup>
</p>
<p>
The above Coefficients have absolutely no experimental basis outside this model. The default numbers have been found to work and give a result that is line with nature. Interestingly, the model is not particularly 
sensitive to the values of these coefficients. Changing the coefficients dramatically will certainly change the results, but does not have a huge impact on how the changes in Greenhouse Gas concentrations may affect the
temperature profile. This is an encouraging result, as it means it does not matter too much if we cannot model convection perfectly, as this is not what we are trying to test. Having a convection model however
brings the profile closer to reality, and hence allows a realistic modelling of Vapour Feedback scenarios.<br/>
A parabolic relationship between the gradients differential ( <sup>&Delta;T</sup>/<sub>&Delta;P</sub> - L<sub>d</sub> / <sub>&#254; g</sub> ) and the flux is based on the idea that the gradient differential drives the force of convection
and also determines the payload - ie the amount of energy transferred per unit of air being convected. 
</p>
<h3>Solar Input</h3>
<p>
As the solar input is not the main thrust of the model, we are just happy for now to simulate the decay of solar flux to the surface in a manner that is not unlike reality. Only the clouds affect 
the solar input.
To simulate the solar input decay, two decay models are applied. Firstly, to simulate the absorption from Ozone at the top of the atmosphere we apply an exponential decay of 8% of the spectrum, such that
99% of this 8% is absorbed in the top 100hPa of atmosphere.
To simulate further decay, mostly from H2O absorption, we will decay another 8% but much slower, such that 99.9% of the 8% is absorbed over an entire atmosphere.
This means that without clouds, the surface will get 16% less solar radiation than the top of the atmosphere.<br/>
With clouds added to the model a certain amount will be absorbed in the cloud layer, some will be reflected and hence lost to the system. What gets through the clouds will be subject to some reflections between 
surface and cloud.
</p>
<p>
The Solar Constant <?=SOLAR_CONSTANT?> Kwm<sup>-2</sup> is divided by four to give an average surface flux for the earth's surface, before attenuation is applied. So the model input is <?=round(SOLAR_CONSTANT/4, 2)?> Kwm<sup>-2</sup>
which is equivalent to having the sun about 15&deg; above the horizon constantly. (ignoring extra attenuation associated with more atmosphere to pass through).

</p>
<h3>Longwave Energy Fluxes</h3>
<p>
The model solves the temperature of the surface and the average temperatures of 10 layers of atmosphere. The energy flux between each layer sets up an equation:<br/>
<br/>
L<sub>u</sub> - L<sub>d</sub> + C = S<br/>
<br/>
L<sub>u</sub> - Longwave Radiation going up from the layers below<br/>
L<sub>d</sub> - Longwave Radiation coming down from the layers above<br/>
C - Energy transferred by convection<br/>
S - the solar (Shortwave Energy) input coming down<br/>
</p>
<p>
As the up and down fluxes are the sums of all the layers above and below, the equation in full is:<br/>
<br/>
L<sub>(s)f</sub>.A<sub>(s)t</sub> + L<sub>(0)f</sub>.A<sub>(0)t</sub> + L<sub>(1)f</sub>.A<sub>(1)t</sub> +<br/>
..  L<sub>(n)f</sub> <br/> 
- L<sub>(n+1)f</sub> <br/>
- L<sub>(n+2)f</sub>.A<sub>(n+2)t</sub> + .. L<sub>(9)f</sub>.A<sub>(9)t</sub><br/>
 = S<sub>(n)i</sub><br/>
<br/>
Where:<br/>
<br/>
L<sub>(i)f</sub> - is the total energy flux from layer i, <sub>(s)</sub> denotes the surface layer<br/>
A<sub>(i)t</sub> - is the attenuation of the radiation from layer i from the layers between it and the surface n.<br/>
S<sub>(n)i</sub> - is the Solar input to layer n.<br/>


</p>
<p>
To solve the upwards and downwards Longwave fluxes we use Plancks Equation to determine the energy at each wavelength interval between 3.2&#181; - 100&#181; .
For each layer of atmosphere and surface the Energy at any particular wavelength is the emisivity(y) * F( y, T ).
<br/>
Where F( y, T ) is <a href="http://en.wikipedia.org/wiki/Planck's_law" target="_blank">Planks Equation</a>.<br/>
<br/>
For the surface, the emisivity is considered constant for every wavelength. For atmosphere layers, the emisivity is equal to the layers ability to absorb at that wavelength.
Only the cloud layer is considered to be able to reflect (disperse) Longwave radiation.<br/>
<br/>
The energy at a particular wavelength from a particular layer once calculated is then attenuated by the absorption of the layers between.
The energy fluxes up and down at a particular surface (between two layers) are calculated (integrated over the entire bandwidth) for every layer including the surface with absorption attenuation.
The energy equation F( y, T )  being non linear, we need to approximate them to a linear form. Using initial temperatures and partical derivatives (how the energy changes per change of Temperature), we can 
construct 11 simulataneous euqations. To solve the simultaneous equations, they are turned into a matrix equation for inversion. The solution gives an improvement solution. To improve the result
again, the matrix is recreated using the new temperatures, and the process is repeated until the solutions begin to converge.
</p>
<h3>Gas Absorption Profiles</h3>
<p>
For this model to function, we require good absorption profiles for the major Greenhouse gasses. Theses can be downloaded from: <a href="http://spectra.iao.ru/1400x865/en/mixt/" target="_blank">http://spectra.iao.ru/1400x865/en/mixt/</a>.
The absorption profiles need to be normalised to atmsopheric concentrations, usually by selecting a profile that is equivalent to 10hPa of atmosphere thickness.<br/>
One of the bizarre behaviours of Gas Absorptions is the way they vary with atmospheric pressure. For this reason we need to acquire for most gasses, a profile representative of the absorption 
spectrum at 0-100hPa of pressure and one at 900-1000hPa of pressure. H2O spectrum has been deduced to 1hPa of thickness (equaivalent to surface conditions of 293&deg; K 50% hunidity) because this has some very strong peaks and we would like some resolution at the top layers where vapour has much 
lower concentration. The spectrum of a gas at any layer then is just assumed to be the weighted average of the two extremes.<br/>
It is very noticeable with H2O vapour and CO2 gas, that with less atmospheric pressure, the absorption bands become more distinct, with narrower/higher peaks and troughs.<br/>
It would be speculated that at height, the forcing effect of increasing these greenhouse gasses becomes less sensitive to change because of this fact.
</p>
<p>
Before flux equations can be deduced, the total absorption/emmisivty of a layer needs to be deduced. This is done by merging all the absorption profiles. The transmisivity for each
wavelength for each gas over that layer is multplied to get a total transmisivity. Where there is cloud, the transmisivity is also reduced by a reflection. The emisivity and non-transmisivity 
diverge in cloudy layers. The emisivity of a cloudy layer (its potential to emulate a black body emitter) is equal to what the layer does not reflect and does not transmit. However, for the purpose of 
calculating the attenutation through this layer, we are interested in only what it does not transmit ( 1 - reflections - absorption ). 

</p>
<h3>Reflections</h3>
<p>
For every direct flux from a layer to a surface, there are also reflections from the ground, cloud, and multiple reflections between ground and cloud. Multiple reflections can be summed using the theory
of <a href="http://en.wikipedia.org/wiki/Geometric_series">Geometrical series</a>. each reflection is attenuated by the albedo of the reflecting surface and the layers in between. For multiple reflections, 
the albedo of both surface and ground and the attenuation of each layer twice becomes to ratio for adding infinite number of reflections. These are obviously quite small.
</p>
<h3>High emisivity Correction</h3>
<p>
The 10 layer model approximates a continuous atmosphere providing the emisivity/absorption of the layers is low and or the temperature gradient is low. 
When the emisivity becomes high, the energy flux from a layer is not representing the average temperature of the layer, but that of a temperature existing 
somewhere between closest surface of the layer and the middle. If the emisivity becomes 1, 
then the layer is no longer transparent to light (of that wavelength) and the outer surface becomes the source of blackbody radiation.</p>
<p>
Between transparency and opaque there is a continuum in which we need to factor. We need to calculate the radiation from the a layer which has a temperature gradient thoughout.
The temperature gradient is factored into the equations by using the temperatures of the layers above and below. The apparent temperature is then a temperature between the layers average temperature
in the middle of the layer and the temperature of the layer adjacent (above for radaition upwards, below for radiation downwards)</p>
<p>
The position between the two temperatures is determined by the following mathematics:</p>
<p>
We assume the temperature gradient is constant, and we assume the radiation output is linear with temperature. This assumption is fine for the small temperature differences between adjacent layers.
</p>
<p>
If we divide the layer into many sublayers, the energy escaping from the layer is now a sum of the many sublayers
</p>
If the total layer emisivity was A, and the number of divisions is N
</p>
<p>
Then emisivity of each sublayer is now a = 1 - ( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup><br/> 
<br/>
The emission from each layer is the emisivity (above) * E(T<sub>a</sub>,y) + <sup>dE</sup>/<sub>dT</sub> * &delta;T <br/>
E(T<sub>a</sub>,y) - being Plancks Equation - a function of temperature and wavelength<br/>
<sup>dE</sup>/<sub>dT</sub> - The change of energy with respect to Temperature<br/>
T<sub>a</sub> - The average temperature of the layer<br/>
&delta;T - the deviation of the temperature of the sublayer from the average temperature<br/>
<br/> 
The energy is absorbed through each layer by attenuating by the transmission for every layer until the surface is reached<br/>
<br/>
For a layer which is j levels in (from the base) the attenuation is:<br/>
(1-a)<sup>N-j</sup> => (1 - {1 - ( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup>})<sup>N-j</sup>  =>  (( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup>)<sup>N-j</sup> => ( 1 - A )<sup>1 - <sup>j</sup>/<sub>N</sub></sup> <br/>
<br/>
Finally the temparature of each sublayer is determined by how far from the centre it is. If the total temperature change from the top of the layer and ther bottom of the layer is
&Delta;T, then the temperature at the jth layer (where j is zero at the bottom surface and is N at the top) is:<br/>
<br/>
T<sub>j</sub> = T<sub>a</sub> + (<sup>j</sup>/<sub>N</sub> - 0.5) * &Delta;T<br/>
&delta;T = (<sup>j</sup>/<sub>N</sub> - 0.5) * &Delta;T<br/>
<br/>
So the energy flux at the top of the layer is the sum:<br/>
<sub>j=1 to j=N</sub><br/>
<br/>
<span style="font-size: 2em">&#931;</span>  { E(T<sub>a</sub>,y) + <sup>dE</sup>/<sub>dT</sub> . (<sup>j</sup>/<sub>N</sub> - 0.5) . &Delta;T } . {1 - ( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup>} . { ( 1 - A )<sup>1 - <sup>j</sup>/<sub>N</sub></sup> }<br/>

OR
<br/>

<span style="font-size: 2em">&#931;</span> E(T<sub>a</sub>,y) . {1 - ( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup>} . { ( 1 - A )<sup>1 - <sup>j</sup>/<sub>N</sub></sup> } <br/>
+<br/>
<sup>dE</sup>/<sub>dT</sub> .  &Delta;T . <span style="font-size: 2em">&#931;</span> (<sup>j</sup>/<sub>N</sub> - 0.5)  . {1 - ( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup>} . { ( 1 - A )<sup>1 - <sup>j</sup>/<sub>N</sub></sup> }
<br/>
<br/>
Now the first part must by logical hand-waving equals the Energy Emission of the layer without a temperature gradient, which is just A . E(T<sub>a</sub>,y). Surely we would find that the expression becomes one of 
those converging infinite series with some more work - but that is not our interest.<br/>
We want to know how much the Energy emissions diverge from the average Temperature result, which is the second part of the expression <br/>
<br/>
<sup>dE</sup>/<sub>dT</sub> .  &Delta;T can be referred to as : &Delta;E, ie the difference in energy from &Delta;T, which was the temperature change from the base to top of the layer.<br/>
<br/>
&Delta;E . <span style="font-size: 2em">&#931;</span> (<sup>j</sup>/<sub>N</sub> - 0.5) . {1 - ( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup>} . { ( 1 - A )<sup>1 - <sup>j</sup>/<sub>N</sub></sup> }<br/>
<br/>
But the expression we need is really only the fraction that the summation creates:<br/>
<br/>
<sub>j=1 to j=N</sub><br/>
C<sub>E</sub> = <span style="font-size: 2em">&#931;</span> (<sup>j</sup>/<sub>N</sub> - 0.5)  . {1 - ( 1 - A )<sup><sup>1</sup>/<sub>N</sub></sup>} . { ( 1 - A )<sup>1 - <sup>j</sup>/<sub>N</sub></sup> }<br/>
</p>
<p>
When a very large number is used for N, ie as the N approaches infinity, the sum converges to a function of Absoprtivity as below:<br/>
At low absorptivity/emisivity the the correction factor is low, ie the average temperature of the layer predicts the radiation emmision very well.
Even as the absorptivity gets high, the correction factor remains low, but as the absorptivity approaches 1.0, the correctiong factor very slowly converges to 0.5.
A correction of 0.5, means that the radiation emmisions reflect the temperature at the facing surface of the layer, like a totally opaque substance. 
</p>
<p>
<embed src="include/process.php?actiontype=getEMCGraph" width="410" height="360"
type="image/svg+xml"/>
</p>
<p>
The slow convergence to 0.5 correction of the above function has implications for the "Saturation Argument" often used by climate skeptics. If the layer is truly absorbing 100% of radiation, 
then it cannot transmit energy in that spectral band. A temperature gradient drives the flux of longwave radiation. In the atmosphere, the layer below being normally warmer will produce more 
radiation than the layer above, however, at 100% absorption, it is the temperature of the top surface of the bottom layer, compared the the lower surface of the above layer - and these have 
the same temperature, whch means that effectively there is now no temperature gradient to be perceived. In reality, this scenario is really predicting that longwave energy transferal of energy grades 
into heat conduction. A substance will never have an absorptivity of 1.0, as solids transfer heat through conduction, which could be thought of as electromagentic vibrations being passed between adjacent
molecules.</p>
<p>
Because of the nature of the Emmisivty Correction Function, it is almost impossible to completely saturate a bandwidth using a greenhouse gas, even between 99% absorption and 99.9% absorption, 
there is some forcing difference. Yet, the saturation argument does still carry some weight, in that the effects of increasing greenhouse gasses does still become less even if it does not converge in a 
true mathematical sense. 

</p>
<p>
The emisivity Correction is applied by multiplying the flux from a layer by (1 - C<sub>E</sub>). Then adding the flux of the layer above or below (depending on direction the flux) by C<sub>E</sub>
to the total flux equation for a flux calculation surface equation.<br/>
<br/>
So now the flux at a surface n equation becomes:<br/>
<br/>
L<sub>(s)f</sub>.A<sub>(s)t</sub> + L<sub>(0)f</sub>.A<sub>(0)t</sub> + L<sub>(1)f</sub>.A<sub>(1)t</sub> +<br/>
..  L<sub>(n)f</sub> *(1 - C<sub>(n)E</sub>) + L<sub>(n+1)f</sub> * C<sub>(n)E</sub><br/> 
- L<sub>(n+1)f</sub> * (1 - C<sub>(n+1)E</sub>) - L<sub>(n)f</sub> * C<sub>(n+1)E</sub><br/>
- L<sub>(n+2)f</sub>.A<sub>(n+2)t</sub> + .. L<sub>(9)f</sub>.A<sub>(9)t</sub><br/>
 = S<sub>(n)i</sub><br/>
<br/>
Where:<br/>
<br/>
L<sub>(i)f</sub> - is the total energy flux from layer i<br/>
C<sub>(i)E</sub> - is the Emmisivty corrections (integrated over all wavelengths)<br/>
A<sub>(i)t</sub> - is the attenuation of the radiation from layer i from the layers between it and the surface n.<br/>
S<sub>(n)i</sub> - is the Solar input to layer n.<br/>
<br/>
The equation above fails to include the corrections to the non adjacent layers, which the model does cover - however, it should be noted that as the correction factors for each layer 
are unlikely to vary much, they are very likely to cancel out, so it is really only the correction factors applied to the adjacent layers that really have much effect.
</p>
<p>
<h3>Emisivity Correction at top of atmosphere?</h3>
<p>
Because the top layer may be thought to grade off yo the temperatures of space - 3&deg; K, applying a correction to the top layer makes sense, even if the changes are too great to be considered linear.
However, wth only ten layers, and the top layer being still so far off from the temperature of space - plus the fact that the thermosphere is actually very hot, it leaves a dilemna. Ideally it would be good 
to apply a full correction to the last layer, which would mean doubling the above correction factor (as the above factor is for grading between layers and converges to 0.5), however, given that the temperature decay to 
space is so non-linear, I have compromised and left just the half correction. This would simulate the last layer grading off to a temperature at the 'top' of the atmosphere that emits half the radiation
of the middle of the layer. This does help the model simulate a slight stratosphere inversion at the top - which is good. Applying a full corection would result in some of the very saturated absorption 
bands emitting absolutely no radiation to space. This is not the case from satellite observations, there are certainly emissions across the longwave spectrum, so it seems that a full correction would be wrong.<br/>
It would seem then that the very top of the atmosphere still has a significant temperature long after the atmosphere loses its ability to block infrared light in its most saturated bands.
 
</p>
<h3>Solving the Equations</h3>
<p>
Using linear approximations of all the energy fluxes, a 11X11 matrix is setup. The matrix is inverted using PHP Pear class Math.Matrix.php, which is the quickest part of the process.
Each time a solution is derived, the model runs the scenario again with the improved temperatures. The Planks energy equations behave quite well using linear approximations, however the convection equations
have a high second derivative, which does slow down the convergence. Making dramatic changes to the atmosphere will mean many corrections before the solutions start to converge. A Bounce Correction is applied
if the results appear to be overshooting and bounce back and forth. A Bounce correction is made by taking a weighted average of the last two results (weighted by the last deviations). This brings the next initial temperatures
closer to the points about which the results appear to be bouncing.
</p>
<h3>Model Limitations</h3>
<p>
The model does not simulate the dynamic nature of heating and cooling of day and night, relying on an averaging of inputs to produce an equilibrium result that should hopefully provide a 
useful average. This does not negate the possibility that dynamic behaviour could enhance or diminish the effects greenhouse gasses.</p>
<p>
The model has only 10 layers. It can be run with more, but the calculation times become cumbersome. Even so, it would require many more layers to simulate the intricate details of the atmosphere, especially
at the upper levels. The emisivity correction helps overcome some of the problems of having too few number of layers, but using the emisivity correction at the top layer is not as easilly justified</p>
<p>
Convection is modelled very simply, there is no allowance for how wet convection probability may vary according to different relative humidities, also there is no allowance for the convection that may occur from the diurnal 
changes to the temperature profile that the equilibrium state cannot model.
<p>
<p>
There is no Thermosphere in the model for a few reasons, one is that there are not enough layers at height to show it. Using layers of equal pressure means that what may be a huge thickness of atmosphere is minuscule for the model because
it represents such a small weight of atmosphere. Also, there is at this stage no modelling of the flux of charged particles hitting this layer or the effect of the magnetic field on such particles. 
</p>
<p>
The solar input method is not affected by changes made to the greenhouse gasses, only the clouds are calculated in the solar input model. The solar attenuation model is very simplified at this stage.
</p>
<p>
The stratosphere inversion only shows weakly - this is because there is not enough layers at height, and not enough resolution in the solar attenuation model to show with detail how and where ozone 
in the stratosphere absorbs light. The fact that some semblance of a stratosphere inversion does show is remarkable given the models limitations.
</p>
<p>
No cloud feedback effects are modelled. This is probably quite obvious - no one can really predict how global warming would affect clouds, so it is best to work with what is simplest.
</p>
<h3>Can this model prove/disprove Anthropogenic Global Warming?</h3>
<p>
The aim has been to test the basic effects of different greenhouse gasses in a simplified situation. If the logic and programming of this model can be *peer-reviewed by making this open-source and critiqued, it could 
be used as a basic starting place for determining what we do know about Anthropogenic Global Warming, and what is speculation.
If this model predicts a certain amount of warming from a doubling of dry greenhouse gasses, then this is a starting point for asking what do the big models do differently, do these extra complexities add better certainty or
just more contention?<br/>
My aim in creating this model is to bring the global warming debate back to its fundamentals - the physical parameters which can be modelled well. My concern with the large models is that 
they are so cumbersome that no-one can really peer-review them properly. There is an argument that man-made global warming is a physical proven reality, however if it requires super-computers and intricate feedback mechanisms to prove it, then it is 
certainly going to be remain open to debate, because very few people are looking under the hood. The debate requires a much simpler benchmark, a physical model that mediocre-experts can understand and contribute to.<br/>
I hope that this model can contribute to a more scientifically sound debate that is not based on trusting what someone says, or little scraps of misinformation from political commentators, but on real reproducible peer-reviewed*
physical modelling.<br/>
<br/>
*peer-reviewed - in the informal sense that many people with some sort of expertise can see and critique.
</p>
