<?
include( 'Matrix.php' );



define( 'NO_LAYERS', 10 );

define( 'SPECTRUM_FILE', 'utils/long-wave-earth.csv' );
define( 'INITIAL_T_PROFILE_FILE', 'utils/temperature_profile.txt' );
define( 'EMMESIVITY_CORRECTION_FILE', 'utils/emmisivity_correctors.txt' );
define( 'TOTAL_ATMOSPHERE', 1000 ); //1000 hPa make one atmosphere
define( 'SIGMA', 5.67 * pow( 10, -8 ) ); ////σ = 5.67×10−8 W m−2 K−4
define( 'SOLAR_CONSTANT', 1.366 * 1000  );   ///1.366 kW/m²

define( 'WIENS_CONSTANT_B', 2.897768551 * pow(10, -3 )  );   ///2.8977685(51)×10−3
define( 'CLOUD_REFLECTION', 0.26 );
define( 'SURFACE_SOLAR_FLUX', SOLAR_CONSTANT * 0.25 * ( 1 - CLOUD_REFLECTION )  );  //0.28 is what gets to surface,


define( 'H_PLANCKS_CONSTANT', 6.62606896 * pow( 10, -34 ) );
define( 'C_SPEED_OF_LIGHT', 2.99792458 * pow( 10, 8 ) );
define( 'E_NATURAL_LOG_BASE', 2.718281 );
define( 'K_BOLTZMANN_CONSTANT', 1.3806504 * pow( 10,-23 ) );


//for convection modelling


define( 'C_SPECIFIC_HEAT_DRY_AIR',   1.006 * 1000 );  //J kg−1 K−1
define( 'R_SPECIFIC_GAS_CONSTANT', 287 );  //J kg−1 K−1
define( 'HEAT_OF_VAPORISATION', 2257*1000 );   //J/kg
define( 'RATIO_OF_MASS_VAPOUR_TO_AIR', (18.02 / 28.57) );

define( 'COEFFICIENT_OF_DRY_CONVECTION', 2500000 ); //kW/m² /  (K/hPa)-2  // This determines the sensitivity of dry convection 
									  // to the (differential of lapse rate - dry adiabatic lapse rate) squared
define( 'COEFFICIENT_OF_WET_CONVECTION', 800000 ); //kW/m² / (K/hPa)-2      // This determines the sensitivity of wet convection 
									  // to the (differential of lapse rate - wet adiabatic lapse rate) squared

function read ($length='255')
{
   if (!isset ($GLOBALS['StdinPointer']))
   {
     $GLOBALS['StdinPointer'] = fopen ("php://stdin","r");
   }
   $line = fgets ($GLOBALS['StdinPointer'],$length);
   return trim ($line);
}


class Model
{
	//no of layers the model will use in calculation. each layer is treated as a porous blackbody that absorbs and re-emmits. Note the porosity of the layer is wavelength dependant.
	//If the layer absorbs 90% at a particalur wavelengthm then it i assumed that it re-emmits up and down with 90% of a blackbody at that wavelength.

	var $noLayers = NO_LAYERS;

	//the absorption spectrums of each gas are assumed to represent amount absorbed in 100hPa of atmosphere, or in the pressure value below. This is not a partial pressure, it is just a section of atmosphere. 

	var $absorptionDataThickness = 100; //value in hPa


	var $surfaceSpectrumInfraRed = array();
	var $surfacealbedo = 0.95;

	var $temperatureLayersToBeSolved = array();
	var $a_mattrix_of_coefficients = array();

	var $linearFitLapseRate = 70;  //70 degrees over entire atmospher  
	var $verbose = false;

	//var $method = "t4coeffs";
	var $method = "linear";

	

	var $title;

	var $resultSet = array();
	var $outputFile = array();


	var $add_dry_convection = 1;
	var $add_wet_convection = 1;
	var $coefficient_of_wet_convection = COEFFICIENT_OF_WET_CONVECTION;
	var $coefficient_of_dry_convection = COEFFICIENT_OF_DRY_CONVECTION;

	function Model()
	{
		if ( !file_exists( SPECTRUM_FILE ) )
			die('Spectrum File Not Found');

		$f = fopen( SPECTRUM_FILE, "r" );

		$_max_e = 0;
		$_max_y_of_e = 0;


		while( $line = fgets( $f, 1024 ) )
		{
			$a = explode( ',', $line );


			$surfaceSpectrumInfraRed[] = array( 'y'=>round( trim($a[0]), 2 ), 'e'=>trim($a[1]) );

			if ( trim($a[1]) > $_max_e )
			{
				$_max_e = (float)(trim($a[1]));
				$_max_y_of_e = (float)( trim($a[0]) );
			}

			$k++;
			
		}


		$this->no_datapoints = sizeof( $surfaceSpectrumInfraRed );
		$this->waveLengthIncrement = ($surfaceSpectrumInfraRed[ sizeof( $surfaceSpectrumInfraRed )-1  ]['y'] - $surfaceSpectrumInfraRed[0]['y'] ) / ($this->no_datapoints-1);
		$this->maxY = $surfaceSpectrumInfraRed[ sizeof( $surfaceSpectrumInfraRed )-1  ]['y'];
		$this->minY = $surfaceSpectrumInfraRed[0]['y'] ;

		$this->maxOutputWavelength = $_max_y_of_e;
		


		for( $k=0; $k<sizeof( $surfaceSpectrumInfraRed ); $k++ )
		{

			if ( $k==0 )
				$_increment = $surfaceSpectrumInfraRed[1]['y'] - $surfaceSpectrumInfraRed[0]['y'];
			else if ( $k == sizeof( $surfaceSpectrumInfraRed )-1 )
				$_increment = $surfaceSpectrumInfraRed[$k]['y'] - $surfaceSpectrumInfraRed[$k-1]['y'];
			else
				$_increment = ($surfaceSpectrumInfraRed[$k+1]['y'] - $surfaceSpectrumInfraRed[$k-1]['y'])/2;

			$surfaceSpectrumInfraRed[$k]['i'] = $_increment;


			$E += $surfaceSpectrumInfraRed[$k]['e'] * $_increment;

		}

		$this->Es = $E;

		//echo "Number of datapoints: ".$this->no_datapoints."\n";
		//echo "Av Wavelength Increment: ".$this->waveLengthIncrement." microns\n";
		//echo "Start Wavelength: ".$this->minY ." microns\n";
		//echo "End Wavelength: ".$this->maxY ." microns\n";
		//echo "Total Upward Flux: ".$E." Wm-2\n";


		$this->surfaceSpectrumInfraRed = $surfaceSpectrumInfraRed;


 		///Wien's displacement law
		

		$this->Ts = WIENS_CONSTANT_B / ($this->maxOutputWavelength * pow(10,-6));
		$this->surfacealbedo = ($this->Es / (SIGMA * pow( $this->Ts , 4 ) )); 


		
		$f = fopen( EMMESIVITY_CORRECTION_FILE, 'r' );
	
		while( $line = fgets($f, 1024) )
		{
			$a = explode(',', rtrim($line) );

			if ( $a[0] == 1 )
			{
				$_index =  'i';
			}
			else
			{	
				$_index =  floor( 0 - round( log( 1 - $a[0], 10 ), 1 ) );
			}			

			$this->emmisivity_correctors[ $_index ][] = array ( 'a'=>$a[0], 'c' => $a[1] );
		}


		fclose( $f );

	}



	function createLayers( $n = NULL )
	{
		$this->outputStatus( 0, 'Creating Layers' );


		if ( $n )
			$this->noLayers = $n;

		if( $this->verbose )
				echo "Normalising Decays and XFade Models\n";
			else
				$this->output( "Normalising Decays and XFade Models\n" );


		$this->normaliseDecays();
		$this->normaliseXFades();

		if  ( file_exists( INITIAL_T_PROFILE_FILE ))
		{
			$f = fopen( INITIAL_T_PROFILE_FILE, 'r' );

			while( $line = fgets($f,1024))
			{
				$a = explode(',', $line );
				
				$a_t_profile[$a[0]] = rtrim($a[1]);

				$max = max( $a[0], $max );

			}

			fclose($f);

			for( $i=0; $i<= $this->noLayers; $i++ )
			{


				$_index_upper = ceil($i*$max/$this->noLayers);
				$_index_lower = floor($i*$max/$this->noLayers);

				if ( $_index_upper != $_index_lower )
				{
					$d = ($a_t_profile[$_index_upper]-$a_t_profile[$_index_lower]);

					$di = $i - $_index_lower*($this->noLayers/$max);
					$dn = $_index_upper*($this->noLayers/$max)  - $_index_lower*($this->noLayers/$max);

					$dt = ($di/$dn) * $d;

					$a_layer_t[$i] = $a_t_profile[$_index_lower] + $dt;
				}
				else
					$a_layer_t[$i] = $a_t_profile[$_index_upper];
			}	

		}


		if ( !$this->resultSet[0] )
		{
			$bool_create_layer_Ts = true;

			if (  $a_layer_t[0] ){
				$this->layers['s'] =  new Layer();
				$this->layers['s']->T = $a_layer_t[0];
			}

			$this->resultSet[0]['values'][0] = $a_layer_t[0];
		}
		else 
		{

			$_last_result_index = sizeof( $this->resultSet ) - 1;


			$this->layers['s'] =  new Layer();
			$this->layers['s']->T = $this->resultSet[$_last_result_index]['values'][0];

		}


		for( $i=1; $i<= $this->noLayers; $i++ )
		{
			if( $this->verbose )
				echo "Calculating Absorption Spectrum: Layer ".($i-1)."\n";
			else
				$this->output( "Calculating Absorption Spectrum: Layer ".($i-1)."\n" );
			

			$this->layers[$i-1] = new Layer();

			$this->layers[ $i-1 ]->absSpectrum = $this->getAbsorptionSpectrum( $i );

			if ( $bool_create_layer_Ts )
			{

				if ( $a_layer_t[$i] ){
					$this->layers[ $i-1 ]->T = $a_layer_t[$i];
				}else{			
					die('No T defined for '.$i );	
				}

				$this->resultSet[0]['values'][$i] = $a_layer_t[$i];
			}
			else if ( !$this->layers[ $i-1 ]->T )
			{
				$this->layers[ $i-1 ]->T = $this->resultSet[$_last_result_index]['values'][$i];
			}

			$this->outputStatus( $i/$this->noLayers, 'Creating Layers' );

		}


		$this->outputStatus( 1, 'Creating Layers' );
		/*
		for( $i=0; $i<=$this->noLayers; $i++ )		
			$this->a_vector_of_solar_constants[] = 	SURFACE_SOLAR_FLUX;

		*/
		
	}

	function getAbsorptionSpectrum( $n )
	{
		//we need to calucalate the absorption rate now for each wavelength with all greenhouse gasses and their respective decays. 

		$a = array();


		for( $j=0; $j<sizeof($this->surfaceSpectrumInfraRed); $j++ )
		{
			$_transmission = 1;

			for( $k=0; $k<sizeof($this->_absorption_models); $k++ )
			{
				
				$_absorption = $this->_absorption_models[$k][$j]['a'];

				//xfade applies a linear factor to the absorption

				if ( isset( $this->_xfade_models[$k][$n]['d'] ) &&  isset( $this->_alpha_xfade_models[$k][$j]['a'] ) )
				{
					$_absorption = ($_absorption * $this->_xfade_models[$k][$n]['d']) + (( 1 - $this->_xfade_models[$k][$n]['d'] ) * $this->_alpha_xfade_models[$k][$j]['a'] ) ;		

				}
				

				$_gas_transmission = 1 - $_absorption;

				
				//decay and scenario apply an exponential factor to simulate alteration to concentration		
				
				$_gas_transmission = pow( $_gas_transmission, ($this->_decay_models[$k][$n]['d'] * $this->scenario_model[$k]) );
				$_gas_transmission = pow( $_gas_transmission, TOTAL_ATMOSPHERE / ( $this->thickness_model[$k]  * $this->noLayers ) );

				$_transmission = $_transmission * $_gas_transmission;

			}

						

			if ( ($n >= ($this->cloudParameters['base']+1) ) & ($n <= ($this->cloudParameters['top']+1) ) )
			{	
				$emmesivity = 	
				(  
                                   $this->cloudParameters['cover'] * 
				   (  
                                       ($this->cloudParameters['longwave']['absorption']) + ( $this->cloudParameters['longwave']['passthru'] * ( 1-$_transmission ) ) 
                                   )
                                ) 
				+ 
				(1 - $this->cloudParameters['cover']) * (1-$_transmission);


				$_transmission = 	
				($this->cloudParameters['cover'] * $this->cloudParameters['longwave']['passthru'] * $_transmission )
				+ 
				(1 - $this->cloudParameters['cover']) * $_transmission;

				//not counting what passes through, what is the absorption
	
			}	

			//just an error throw
			if ( $_transmission < 0 || $_transmission > 1 )
			{
				echo "Point $j tramsimission: $_transmission ".$this->surfaceSpectrumInfraRed[$j]['y']."\n";
	
				for( $k=0; $k<sizeof($this->_absorption_models); $k++ )
				{
					echo "Model $k ".$this->_absorption_models[$k][$j]['a']."\n";
				}

				exit;
			}		

			$a[$j] = array( 'y'=> $this->surfaceSpectrumInfraRed[$j]['y'], 'a'=> (1-$_transmission) );

			if ( isset( $emmesivity ) )
			{
				$a[$j]['e'] = $emmesivity;
				unset( $emmesivity );
			}

			
		}

		return $a;

	}		

	function normaliseDecays()
	{	
		//noramlise the decay curves to a value for each layer


		for( $k=1; $k<=$this->noLayers; $k++ )
		{
			for( $j=0; $j<sizeof($this->decay_models); $j++ )
			{
				for( $i=0; $i<sizeof( $this->decay_models[$j] ); $i++ )
				{
					$_normalise =  $this->noLayers / $this->decay_models[$j][ sizeof( $this->decay_models[$j] ) - 1 ]['n'];
	

					if ( ($this->decay_models[$j][$i]['n'] * $_normalise)  == $k )
					{
						$this->_decay_models[$j][$k] = array( 'n'=> $k, 'd'=>  $this->decay_models[$j][$i]['d'] );
					}
					else 
					{
						if ( $i>0 )
						{
							//echo 'interpolating for '.$this->surfaceSpectrumInfraRed[$k]['y']."\n";
							//echo 'between '.$this->absorption_models[$j][$i-1]['y'].' & '.$this->absorption_models[$j][$i]['y']."\n";

							if(  ( $k > ($this->decay_models[$j][$i-1]['n' ]*$_normalise))  &&  ( $k < ($this->decay_models[$j][$i]['n']*$_normalise) ) )
							{
								//interpolate the value for $k point
								$_decay = $this->decay_models[$j][$i-1]['d'] +
						
									($this->decay_models[$j][$i]['d'] - $this->decay_models[$j][$i-1]['d'])
									*
 									(($k - ($this->decay_models[$j][$i-1]['n']*$_normalise) ) 
									/ 
									(($this->decay_models[$j][$i]['n'] - $this->decay_models[$j][$i-1]['n'])*$_normalise) );
				

								$this->_decay_models[$j][$k] = array( 'n'=> $k, 'd'=>  $_decay );

								break;
							}
						}


						if (  $i == 0  )
						{ 	
							//if the absorption spectrum begins after y(k)
					
							if ( $n < ($this->decay_models[$j][0]['n']*$_normalise) )
							{

								$this->_decay_models[$j][$k] = array( 'n'=> $k, 'd'=>  $this->decay_models[$j][0]['d'] );
							}
						}
						else if ( $i ==  sizeof( $this->decay_models[$j] )-1 )
						{
							//if the absorption spectrum ends before y(k)
					
							if ( $n > ($this->decay_models[$j][$i]['n']*$_normalise) )
							{

								$this->_decay_models[$j][$k] = array( 'n'=> $k, 'd'=>  $this->decay_models[$j][$i]['d'] );
							}

						}
						

					}
				}
			}
		}

	}

	function normaliseXFades()
	{	
		//noramlise the decay curves to a value for each layer


		for( $k=1; $k<=$this->noLayers; $k++ )
		{
			for( $j=0; $j<sizeof($this->decay_models); $j++ )
			{
				if ( !$this->xfade_models[$j] )
					continue;

				for( $i=0; $i<sizeof( $this->xfade_models[$j] ); $i++ )
				{
					$_normalise =  $this->noLayers / $this->xfade_models[$j][ sizeof( $this->xfade_models[$j] ) - 1 ]['n'];
	

					if ( ($this->xfade_models[$j][$i]['n'] * $_normalise)  == $k )
					{
						$this->_xfade_models[$j][$k] = array( 'n'=> $k, 'd'=>  $this->xfade_models[$j][$i]['d'] );
					}
					else 
					{
						if ( $i>0 )
						{
							//echo 'interpolating for '.$this->surfaceSpectrumInfraRed[$k]['y']."\n";
							//echo 'between '.$this->absorption_models[$j][$i-1]['y'].' & '.$this->absorption_models[$j][$i]['y']."\n";

							if(  ( $k > ($this->xfade_models[$j][$i-1]['n' ]*$_normalise))  &&  ( $k < ($this->xfade_models[$j][$i]['n']*$_normalise) ) )
							{
								//interpolate the value for $k point
								$_decay = $this->xfade_models[$j][$i-1]['d'] +
						
									($this->xfade_models[$j][$i]['d'] - $this->xfade_models[$j][$i-1]['d'])
									*
 									(($k - ($this->xfade_models[$j][$i-1]['n']*$_normalise) ) 
									/ 
									(($this->xfade_models[$j][$i]['n'] - $this->xfade_models[$j][$i-1]['n'])*$_normalise) );
				

								$this->_xfade_models[$j][$k] = array( 'n'=> $k, 'd'=>  $_decay );

								break;

							}
						}


						if (  $i == 0  )
						{ 	
							//if the absorption spectrum begins after y(k)
					
							if ( $n < ($this->xfade_models[$j][0]['n']*$_normalise) )
							{

								$this->_xfade_models[$j][$k] = array( 'n'=> $k, 'd'=>  $this->xfade_models[$j][0]['d'] );
							}
						}
						else if ( $i ==  sizeof( $this->xfade_models[$j] )-1 )
						{
							//if the absorption spectrum ends before y(k)
					
							if ( $n > ($this->xfade_models[$j][$i]['n']*$_normalise) )
							{

								$this->_xfade_models[$j][$k] = array( 'n'=> $k, 'd'=>  $this->decay_models[$j][$i]['d'] );
							}

						}
						

					}
				}
			}
		}

	}


	function loadAbsorptionSpectrums( $a )
	{

		$this->outputStatus( 0, 'Loading Absorption Data' );

		for( $k=0; $k<sizeof( $a ); $k++ )
		{
			if( $this->verbose )
				echo "Loading ".$a[$k]['alpha']." into absorptions model\n";
			else
				$this->output( "Loading ".$a[$k]['alpha']." into absorptions model\n" );

			$_scale = ( $a[$k]['scale'] )?$a[$k]['scale']:1;

			$f = @fopen( $a[$k]['alpha'], 'r' );

			if(!$f)
			{
				if( $this->verbose ){
					die( "file: ".$a[$k]['alpha']." not found");	
				}else{
					$this->output( "file: ".$a[$k]['alpha']." not found"); exit;
				}
			}

			while( $line = fgets( $f, 1024 ) )
			{
				if ( substr( $line,0, 2 ) == '//' )
					continue;

				$a_ = explode( ',', $line );

				$this->absorption_models[$k][] = array( 'y'=>trim($a_[0]), 'a'=>(trim($a_[1])/$_scale) );
			}


			fclose( $f );

			if( $this->verbose )
				echo "Loading ".$a[$k]['decay']." into decay models\n";
			else
				$this->output( "Loading ".$a[$k]['decay']." into decay models\n" );

			$f = @fopen( $a[$k]['decay'], 'r' );

			if(!$f)
			{
				if( $this->verbose ){
					die( "file: ".$a[$k]['decay']." not found");
				}else{
					$this->output( "file: ".$a[$k]['decay']." not found"); exit;
				}
			}

			while( $line = fgets( $f, 1024 ) )
			{
				$a_ = explode( ',', $line );

				$this->decay_models[$k][] = array( 'n'=>trim($a_[0]), 'd'=>trim($a_[1]) );
			}

			fclose( $f );

			if ( $a[$k]['xfade'] )
			{
				$f = @fopen( $a[$k]['xfade'], 'r' );

				if(!$f)
				{
					if( $this->verbose ){
						die( "file: ".$a[$k]['xfade']." not found" );
					}else{
						$this->output( "file: ".$a[$k]['xfade']." not found"); exit;
					}
				}

				while( $line = fgets( $f, 1024 ) )
				{
					$a_ = explode( ',', $line );

					$this->xfade_models[$k][] = array( 'n'=>trim($a_[0]), 'd'=>trim($a_[1]) );
				}

				fclose( $f );

			}

			if ( $a[$k]['alpha_xfade'] )
			{
				$f = @fopen( $a[$k]['alpha_xfade'], 'r' );
					
				if(!$f)
				{
					if( $this->verbose ){
						die( "file: ".$a[$k]['alpha_xfade']." not found");	
					}else{
						die( "file: ".$a[$k]['alpha_xfade']." not found");	
					}
				}

				while( $line = fgets( $f, 1024 ) )
				{
					if ( substr( $line,0, 2 ) == '//' )
						continue;

					$a_ = explode( ',', $line );

					$this->alpha_xfade_models[$k][] = array( 'y'=>trim($a_[0]), 'a'=>(trim($a_[1])/$_scale) );
				}

				fclose( $f );

			}	

			if ( $a[$k]['thickness'] )
				$this->thickness_model[$k] = $a[$k]['thickness'];
			else
				$this->thickness_model[$k] = $this->absorptionDataThickness;

			//here we can apply the multiplication of the GHG
			if ( $a[$k]['scenario'] )
				$this->scenario_model[$k] = $a[$k]['scenario'];
			else
				$this->scenario_model[$k] = 1;
			
		}

		//we need to interpolate the absorptions to the primary wavelengths of the surface emissions

		if( $this->verbose )
			echo "Adding and normalising absorption curves\n";
		else
			$this->output( "Adding and normalising absorption curves\n" );

		for( $j=0; $j<sizeof($this->absorption_models); $j++ )
			$_start_at[$j] = 0;

		$size_of_spectrum = sizeof( $this->surfaceSpectrumInfraRed );

		for( $k=0; $k<$size_of_spectrum; $k++ )
		{
			if( $_index_verbose > 100  )
			{
				if ( $this->verbose )
					echo "Calculating $k point (".$this->surfaceSpectrumInfraRed[$k]['y'].")\n";
				else
					$this->output( "Calculating $k point (".$this->surfaceSpectrumInfraRed[$k]['y'].")\n" );

				$_index_verbose = 0;

				$this->outputStatus( ($k-1)/(2*$size_of_spectrum), 'Loading Absorption Data' );
			}	

			$_index_verbose++;

			$this->surfaceSpectrumInfraRed[$k]['y'];

			for( $j=0; $j<sizeof($this->absorption_models); $j++ )
			{

				for( $i=$_start_at[$j]; $i<sizeof( $this->absorption_models[$j] ); $i++ )
				{
					//echo $j.' : '.$i." ".$this->surfaceSpectrumInfraRed[$k]['y']." : ".$this->absorption_models[$j][$i]['y']."\n";

					if ( $this->absorption_models[$j][$i]['y'] == $this->surfaceSpectrumInfraRed[$k]['y'] )
					{
						$this->_absorption_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $this->absorption_models[$j][$i]['a'] );

						$_start_at[$j] = $i;
						
						//echo "found ".$this->surfaceSpectrumInfraRed[$k]['y']." : ".$this->absorption_models[$j][$i]['y']."\n";

						break 1;
					}
					else 
					{
						if ( $i>0  && $i < sizeof( $this->absorption_models[$j] )-1)
						{
							//echo 'here 2'."\n";

							//echo 'interpolating for '.$this->surfaceSpectrumInfraRed[$k]['y']."\n";
							//echo 'between '.$this->absorption_models[$j][$i-1]['y'].' & '.$this->absorption_models[$j][$i]['y']."\n";

							if(  ($this->surfaceSpectrumInfraRed[$k]['y'] > $this->absorption_models[$j][$i-1]['y'])  &&  ($this->surfaceSpectrumInfraRed[$k]['y'] < $this->absorption_models[$j][$i]['y']) )
							{

								//interpolate the value for $k point
								$_alpha = $this->absorption_models[$j][$i-1]['a'] +
						
									($this->absorption_models[$j][$i]['a'] - $this->absorption_models[$j][$i-1]['a'])
									*
 									(($this->surfaceSpectrumInfraRed[$k]['y'] - $this->absorption_models[$j][$i-1]['y'] ) 
									/ 
									($this->absorption_models[$j][$i]['y'] - $this->absorption_models[$j][$i-1]['y']));
				

								$this->_absorption_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $_alpha );

						
								//echo 'here: '.$this->surfaceSpectrumInfraRed[$k]['y'].' : '.$_alpha."\n";
								$_start_at[$j] = $i-1;
								break 1;

							}
							
						}
						
						if (  $i == 0  )
						{ 
							//echo 'here 3'."\n";
	
							//if the absorption spectrum begins after y(k)
					
							if ( $this->surfaceSpectrumInfraRed[$k]['y'] < $this->absorption_models[$j][0]['y'] )
							{

								$this->_absorption_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $this->absorption_models[$j][0]['a'] );
								break;
							}

						}
						else if ( $i ==  sizeof( $this->absorption_models[$j] )-1 )
						{
							//echo 'here 4'."\n";

							//if the absorption spectrum ends before y(k)
					
							if ( $this->surfaceSpectrumInfraRed[$k]['y'] > $this->absorption_models[$j][$i]['y'] )
							{

								$this->_absorption_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $this->absorption_models[$j][$i]['a'] );
							}

						}
					}
				}

			}

		}

		for( $j=0; $j<sizeof($this->absorption_models); $j++ )
			$_start_at[$j] = 0;

		$_index_verbose = 0;

		for( $k=0; $k<sizeof( $this->surfaceSpectrumInfraRed ); $k++ )
		{
			if( $_index_verbose > 100  )
			{
				
			
				$this->outputStatus( 0.5 + ($k-1)/(2*$size_of_spectrum), 'Loading Absorption Data - upper levels' );

				if ( $this->verbose )
					echo "Calculating $k point (xfade) (".$this->surfaceSpectrumInfraRed[$k]['y'].")\n";
				else
					$this->output( "Calculating $k point (xfade) (".$this->surfaceSpectrumInfraRed[$k]['y'].")\n" );

				$_index_verbose = 0;	
			}
	

			$_index_verbose++;

			//load up xfade alpha file

			for( $j=0; $j<sizeof($this->absorption_models); $j++ )
			{
				if ( !$this->alpha_xfade_models[$j] )
					continue;

				for( $i=$_start_at[$j]; $i<sizeof( $this->alpha_xfade_models[$j] ); $i++ )
				{
					if ( $this->alpha_xfade_models[$j][$i]['y'] == $this->surfaceSpectrumInfraRed[$k]['y'] )
					{
						$this->_alpha_xfade_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $this->alpha_xfade_models[$j][$i]['a'] );
						$_start_at[$j] = $i-1;
						break;
					}
					else 
					{
						if ( $i>0 )
						{
							//echo 'interpolating for '.$this->surfaceSpectrumInfraRed[$k]['y']."\n";
							//echo 'between '.$this->absorption_models[$j][$i-1]['y'].' & '.$this->absorption_models[$j][$i]['y']."\n";

							if(  ($this->surfaceSpectrumInfraRed[$k]['y'] > $this->alpha_xfade_models[$j][$i-1]['y'])  &&  ($this->surfaceSpectrumInfraRed[$k]['y'] < $this->alpha_xfade_models[$j][$i]['y']) )
							{
								//interpolate the value for $k point
								$_alpha = $this->alpha_xfade_models[$j][$i-1]['a'] +
						
									($this->alpha_xfade_models[$j][$i]['a'] - $this->alpha_xfade_models[$j][$i-1]['a'])
									*
 									(($this->surfaceSpectrumInfraRed[$k]['y'] - $this->alpha_xfade_models[$j][$i-1]['y'] ) 
									/ 
									($this->alpha_xfade_models[$j][$i]['y'] - $this->alpha_xfade_models[$j][$i-1]['y']));
				

								$this->_alpha_xfade_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $_alpha );

						
								//echo 'here: '.$this->surfaceSpectrumInfraRed[$k]['y'].' : '.$_alpha."\n";
								$_start_at[$j] = $i-1;
								break;

							}
							
						}
						else if (  $i == 0  )
						{ 	
							//if the absorption spectrum begins after y(k)
					
							if ( $this->surfaceSpectrumInfraRed[$k]['y'] < $this->alpha_xfade_models[$j][0]['y'] )
							{
								$this->_alpha_xfade_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $this->alpha_xfade_models[$j][0]['a'] );
								break;
							}
						}
						else if ( $i ==  sizeof( $this->alpha_xfade_models[$j] )-1 )
						{
							//if the absorption spectrum ends before y(k)
					
							if ( $this->surfaceSpectrumInfraRed[$k]['y'] > $this->alpha_xfade_models[$j][$i]['y'] )
							{

								$this->_alpha_xfade_models[$j][$k] = array( 'y'=> $this->surfaceSpectrumInfraRed[$k]['y'], 'a'=>  $this->alpha_xfade_models[$j][$i]['a'] );
							}

						}
					}
				}
			}

		}

		$this->outputStatus( 1, 'Loading Absorption Data' );

	}

	function createOutgoingSpectrum()
	{
		for( $k=0; $k<sizeof($this->surfaceSpectrumInfraRed); $k++ )
		{
			$spectrum[$k] = array( 'y'=>$this->surfaceSpectrumInfraRed[$k]['y'], 'e'=>0 );

			for( $layer=-1; $layer<$this->noLayers; $layer++ )
			{
				$_layer = ($layer==-1)?'s':$layer;				

				$E = $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[ $_layer ]->T );
				$Ed = $E;

				if ( $_layer !== 's' )
				{
					//correct for the T gradient for HIGH Emmisivity

					$emc = 	$this->getHighEmmissivityCorrection( $this->layers[ $layer ]->absSpectrum[$k]['a'] );

					$E = ( 1 - $emc ) * $E;
					
					if ( $_layer < $this->noLayers-1 )
					{
						if ( $this->layers[ $_layer+1 ]->absSpectrum[$k]['e'] )
						{
							$E = $E + ($emc * $this->layers[ $_layer+1 ]->absSpectrum[$k]['e'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[ $_layer+1 ]->T ) );
						}
						else
						{
							$E = $E + ($emc * $this->layers[ $_layer+1 ]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[ $_layer+1 ]->T ) );
						}
					}

					
					if ( $this->layers[$_layer]->absSpectrum[$k]['e'] )
					{
						$E = $E * $this->layers[$_layer]->absSpectrum[$k]['e'];
						$Ed = $Ed * $this->layers[$_layer]->absSpectrum[$k]['e'];
					}
					else
					{
						$E = $E * $this->layers[$_layer]->absSpectrum[$k]['a'];
						$Ed = $Ed * $this->layers[$_layer]->absSpectrum[$k]['a'];
					}

				}				
				else if ( $_layer=='s' )
				{
					$E = $E * $this->surfacealbedo;
				}

				//get surface reflections
				

				if ( $_layer !== 's' )
				{
					$Ed = ( 1 - $emc ) * $Ed;
					
					if ( $_layer > 0 )
					{
						if ( $this->layers[$_layer-1]->absSpectrum[$k]['e'] )
						{	
							$Ed = $Ed + ( $emc * $this->layers[$_layer-1]->absSpectrum[$k]['e'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[ $_layer-1 ]->T ) );
						}
						else
						{
							$Ed = $Ed + ( $emc * $this->layers[$_layer-1]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[ $_layer-1 ]->T ) );
						}
					}
					else
					{
						if ( $this->layers[$_layer]->absSpectrum[$k]['e'] )
						{
							$Ed = $Ed + ( $emc * $this->layers[$_layer]->absSpectrum[$k]['e'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T ) );
						}
						else
						{
							$Ed = $Ed + ( $emc * $this->layers[$_layer]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T ) );
						}
					}

					


					for( $g=0; $g<$layer; $g++ )
					{
						if ( $g == $layer )
						{
							$Ed = $Ed * (1-$this->layers[ $g ]->absSpectrum[$g]['a']);
						}
						else
						{
							$Ed = $Ed * pow( (1-$this->layers[ $g ]->absSpectrum[$g]['a']), 2 );
						}
					}

					
					
				} 
				else
					$Ed = 0;

				if ( $this->cloudParameters['base'] > $layer )
				{
					$Ec2 = $Ed * $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];
					$Ec1 = $E * $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];

					$at_cycle = $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'] * $this->surfacealbedo;

					for( $g=0; $g<$this->cloudParameters['base']; $g++ )
					{
							
						$Ec2 = $Ec2 * pow( (1-$this->layers[ $g ]->absSpectrum[$g]['a']), 2 );
						$Ec1 = $Ec1 * pow( (1-$this->layers[ $g ]->absSpectrum[$g]['a']), 2 );

						$at_cycle = $at_cycle * pow( (1-$this->layers[ $g ]->absSpectrum[$g]['a']), 2 );
							
					}
						
					$E += $Ec2 * ( 1/ (1-$at_cycle)  );
					$E += $Ec1 * ( 1/ (1-$at_cycle)  );
						
				}
				else if (  $this->cloudParameters['top'] < $layer  )
				{
					$Ec1 = $Ed * $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];

					for( $g=$this->cloudParameters['top']; $g<$layer; $g++ )
					{
						if ( $g == $layer )
						{	
							$Ec1 = $Ec1 * (1-$this->layers[ $g ]->absSpectrum[$g]['a']);
						}
						else
						{
							$Ec1 = $Ec1 * pow( (1-$this->layers[ $g ]->absSpectrum[$g]['a']), 2 );
						}							
					}
				
					$E += $Ec1;					

				}
				

				//now diminish energy to space

				for( $g=$layer+1; $g<$this->noLayers; $g++ )
				{
					$E = $E * (1-$this->layers[ $g ]->absSpectrum[$k]['a']);
					
				}

				$spectrum[$k]['e'] += $E;
			}
		}

		return $spectrum;
	}

	function deduceEquationUsingLinear( $n )
	{
		//inputs from surface
		//all radiation going up
		
		//surface variable
		//$this->temperatureLayersToBeSolved[0];

		if( $this->verbose )
			echo "Deducing flux inputs for N $n \n";
		else
			$this->output( "Deducing flux inputs for N $n \n" );	
		

		//How much of the surface raditation makes it to the nth layer

		$this->outputStatus( $n/($this->noLayers+1), 'Deducing Flux Equations' );



		for( $k=0; $k<sizeof($this->surfaceSpectrumInfraRed); $k++ )
		{	

			if ( !( $_energy_per_t = $this->_energy_cache[0][ $k ]['energy_per_t']))
			{
				//deltaT can be expressed as T - T1
				//Energy = E(T1) + ( T - T1 ) * dE/dT
				//The constant part is therefore:  E(T1) - dE/dT*Ti

				$_energy_per_t = $this->surfacealbedo * $this->surfaceSpectrumInfraRed[$k]['i'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T );
				$_energy_constant = $this->surfacealbedo * $this->surfaceSpectrumInfraRed[$k]['i'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T ) - ( $this->layers['s']->T * $_energy_per_t );

				$this->_energy_cache[0][ $k ]['energy_per_t'] = $_energy_per_t;
				$this->_energy_cache[0][ $k ]['energy_constant'] = $_energy_constant;

			}
			else
				$_energy_constant =  $this->_energy_cache[0][ $k ]['energy_constant'];
			

			//now absorb this energy for each layer in between

			for( $i=0; $i<$n; $i++ )
			{
				$_energy_per_t = $_energy_per_t * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
				$_energy_constant = $_energy_constant * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
			}

			$_total_surface_radiation['constant'] +=  $_energy_constant;
			$_total_surface_radiation['perdeltat'] +=  $_energy_per_t;


			
			//Calculate cloud reflections

			if ( $this->cloudParameters )
			{
				if ( $n <= $this->cloudParameters['base'] )
				{

					//we are under the cloud, so calculate
					$_1st_down_ratio = $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];
					$_1st_up_ratio = $this->surfacealbedo * $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];
					
					for( $i=0; $i<$this->cloudParameters['base']; $i++ )
					{
						$_1st_up_ratio = $_1st_up_ratio * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );

						if ( $i >= $n )
						{
							$_1st_down_ratio = $_1st_down_ratio * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2);
						}
						
					}					
					
					//Sum the infinite series
					// Sum = a / 1 -r
					//$_cloud_reflections_up =  $_1st_up_ratio( 1 + $_1st_up_ratio + $_1st_up_ratio^2 + $_1st_up_ratio^3 ... );
					$_cloud_reflections_up = $_1st_up_ratio * ( 1 / (1 - $_1st_up_ratio ) );

					//$_cloud_reflections_down = $_1st_down_ratio( 1 + $_1st_up_ratio + $_1st_up_ratio^2 + $_1st_up_ratio^3 ... );
					$_cloud_reflections_down = $_1st_down_ratio * ( 1 / (1 - $_1st_up_ratio ) );

					//start with the values calculated so far

					$_total_surface_radiation['constant'] +=  ( $_energy_constant * ($_cloud_reflections_up - $_cloud_reflections_down) );
					$_total_surface_radiation['perdeltat'] +=  ( $_energy_per_t * ($_cloud_reflections_up -	$_cloud_reflections_down) ); 

				
	
				}
				else if (  $n > $this->cloudParameters['top']  )
				{
					//surface to cloud bounces prior to penetrating cloud layer

					$_1st_up_ratio = $this->surfacealbedo * $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];

					for( $i=0; $i<$this->cloudParameters['base']; $i++ )
					{
						$_1st_up_ratio = $_1st_up_ratio * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
					}

					$_total_surface_radiation['constant'] +=  $_energy_constant * $_1st_up_ratio * ( 1 / 1-$_1st_up_ratio ) ;
					$_total_surface_radiation['perdeltat'] +=  $_energy_per_t * $_1st_up_ratio * ( 1 / 1-$_1st_up_ratio ); 


				}

			}

			

		}		
		
		$this->outputStatus(  ( $n/($this->noLayers+1)) + ( 1 / ( pow(($this->noLayers+1),2) )), 'Deducing Flux Equations' );
					

		$a_mattrix['flux'][ 0 ] = $a_mattrix['flux'][ 0 ] + $_total_surface_radiation['perdeltat'];

		//start the solution vector	
		$this->a_vector_of_linear_fit[$n] = ($this->getSolarFlux( $n )) - $_total_surface_radiation['constant'];
		

		if( $this->verbose )
			echo "Layer S flux coefficient: (".$_total_surface_radiation['perdeltat']." * T) + ".$_total_surface_radiation['constant']."\n";
		else
			$this->output( "Layer S flux coefficient: (".$_total_surface_radiation['perdeltat']." * T) + ".$_total_surface_radiation['constant']."\n" );

		//now do one for each layer
		

		for( $layer = 0; $layer< $this->noLayers; $layer++ )
		{
			
				//calculate radiation from layer reaching n
			
				$_total_surface_radiation['constant'] = 0;
				$_total_surface_radiation['perdeltat'] = 0;

				$_total_surface_radiation_above['constant'] = 0;
				$_total_surface_radiation_above['perdeltat'] = 0;

				$_total_surface_radiation_below['constant'] = 0;
				$_total_surface_radiation_below['perdeltat'] = 0;
				
				for( $k=0; $k<sizeof($this->surfaceSpectrumInfraRed); $k++ )
				{
					//When emmisivity is high, the radiation level is not a reflection of the average layere temperature, but of a position off center
					//The correction tells us how much to apportion the current layer, and how mucg to aportion to the neighbouring layer

					if ( $this->layers[ $layer ]->absSpectrum[$k]['a'] > 1 )
						die( "layer $layer point $k ".$this->layers[ $layer ]->absSpectrum[$k]['a'] );

					//if ( $layer < $this->noLayers-1 )
						$_emmisivity_correction = $this->getHighEmmissivityCorrection( $this->layers[ $layer ]->absSpectrum[$k]['a'] );	
					//else
					//	$_emmisivity_correction = 0;

					//how to deal with the top layer?
					//this should be representative of stratosphere, so assume no gradient, and no need for emmisivity correction


					//the portion of energy from layer					
					if ( !isset( $this->_energy_cache[$layer+1][ $k ]['energy_per_t']) )
					{
						//deltaT can be expressed as T - T1
						//Energy = E(T1) + ( T - T1 ) * dE/dT
						//The constant part is therefore:  E(T1) - dE/dT*Ti

						//some layers with cloud reflections have relections, which means absorption and emmisivity are different
						//a will always mean total not getting through, ie reflected + absorption
						//in this case e will represent the emmisivity

						if ( isset( $this->layers[ $layer ]->absSpectrum[$k]['e'] ) )
						{
							$_energy_per_t = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['e'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer]->T );
							$_energy_constant = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['e'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer]->T ) - ( $this->layers[$layer]->T * $_energy_per_t );

						}
						else	
						{
							$_energy_per_t = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['a'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer]->T );
							$_energy_constant = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer]->T ) - ( $this->layers[$layer]->T * $_energy_per_t );
						}


						$this->_energy_cache[ $layer+1 ][ $k ]['energy_per_t'] = $_energy_per_t;
						$this->_energy_cache[ $layer+1 ][ $k ]['energy_constant'] = $_energy_constant;
						
											

					}
					else
					{
						$_energy_constant =  $this->_energy_cache[$layer+1][ $k ]['energy_constant'];
						$_energy_per_t = $this->_energy_cache[$layer+1][ $k ]['energy_per_t'];
						
					}
					
					//get amounts from the layer below
					

					if ( $layer == 0 )
					{
						//the layer previously is the surface, we want the temperature differential, but not the albedo properties
						//we will use the albedo properties of the first layer
						
						if ( !isset( $this->_energy_cache['b'][ $k ]['energy_per_t'] ) )
						{

							if ( isset( $this->layers[ $layer ]->absSpectrum[$k]['e'] ) )
							{
								$_energy_per_t_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['e'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T );
								$_energy_constant_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['e'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T ) - ( $this->layers['s']->T * $_energy_per_t_below );

							}
							else	
							{
								$_energy_per_t_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['a'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T );
								$_energy_constant_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer ]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T ) - ( $this->layers['s']->T * $_energy_per_t_below );
							}

							$this->_energy_cache['b'][ $k ]['energy_per_t'] = $_energy_per_t_below;
							$this->_energy_cache['b'][ $k ]['energy_constant'] = $_energy_constant_below;
						}
						else
						{
							$_energy_per_t_below = $this->_energy_cache['b'][ $k ]['energy_per_t'];
							$_energy_constant_below = $this->_energy_cache['b'][ $k ]['energy_constant'];
						}

					}
					else
					{		
						
						if ( isset( $this->_energy_cache[$layer][ $k ]['energy_per_t'] ) )
						{
							$_energy_per_t_below = $this->_energy_cache[$layer][ $k ]['energy_per_t'];
							$_energy_constant_below = $this->_energy_cache[$layer][ $k ]['energy_constant'];
						}
						else
						{
							if ( isset( $this->layers[ $layer-1 ]->absSpectrum[$k]['e'] ) )
							{
								$_energy_per_t_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer-1 ]->absSpectrum[$k]['e'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer-1]->T );
								$_energy_constant_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer-1 ]->absSpectrum[$k]['e'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer-1]->T ) - ( $this->layers[$layer-1]->T * $_energy_per_t_below );

							}
							else	
							{
								$_energy_per_t_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer-1 ]->absSpectrum[$k]['a'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer-1]->T );
								$_energy_constant_below = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer-1 ]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer-1]->T ) - ( $this->layers[$layer-1]->T * $_energy_per_t_below );
							}


							$this->_energy_cache[$layer][ $k ]['energy_per_t'] = $_energy_per_t_below;
							$this->_energy_cache[$layer][ $k ]['energy_constant'] = $_energy_constant_below;
							

						}
					}

					//get amounts from the layer above
					

					if ( $layer < $this->noLayers-1 )
					{
						if ( isset( $this->_energy_cache[$layer+2][ $k ]['energy_per_t'] ) )
						{
							$_energy_per_t_above = $this->_energy_cache[$layer+2][ $k ]['energy_per_t'];
							$_energy_constant_above = $this->_energy_cache[$layer+2][ $k ]['energy_constant'];
						}
						else
						{
							if ( isset( $this->layers[ $layer+1 ]->absSpectrum[$k]['e'] ) )
							{
								$_energy_per_t_above = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer+1 ]->absSpectrum[$k]['e'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer+1]->T );
								$_energy_constant_above = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer+1 ]->absSpectrum[$k]['e'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer+1]->T ) - ( $this->layers[$layer+1]->T * $_energy_per_t_above );

							}
							else	
							{
								$_energy_per_t_above = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer+1 ]->absSpectrum[$k]['a'] * $this->getPartialDerivative_dEdT_FromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer+1]->T );
								$_energy_constant_above = $this->surfaceSpectrumInfraRed[$k]['i'] * $this->layers[ $layer+1 ]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer+1]->T ) - ( $this->layers[$layer+1]->T * $_energy_per_t_above );
							}

							$this->_energy_cache[$layer+2][ $k ]['energy_per_t'] = $_energy_per_t_above;
							$this->_energy_cache[$layer+2][ $k ]['energy_constant'] = $_energy_constant_above;
							

						}


					}
					
	
					
					//from each layer there is a direct energy and the energy reflected by the surface
					$_energy_per_t_flux_direct = $_energy_per_t;
					$_energy_per_t_flux_reflection = $_energy_per_t * (1-$this->surfacealbedo);

					$_energy_constant_flux_direct = $_energy_constant;
					$_energy_constant_flux_reflection = $_energy_constant * (1-$this->surfacealbedo);

					
					//but if emmisivity is high, we take a portion of this layer
					$_energy_per_t_flux_direct = (1 - $_emmisivity_correction) * $_energy_per_t_flux_direct;
					$_energy_constant_flux_direct = (1 - $_emmisivity_correction) * $_energy_constant_flux_direct;
					$_energy_per_t_flux_reflection = (1 - $_emmisivity_correction) * $_energy_per_t_flux_reflection;
					$_energy_constant_flux_reflection = (1 - $_emmisivity_correction) * $_energy_constant_flux_reflection;

					
					//there is certain amount from the layers above and below
					//to account for the fact that the output of the layer is not really that of the average Temperature of the layer
					//when layer absorption is high
					$_energy_per_t_flux_direct_layer_above = $_emmisivity_correction * $_energy_per_t_above;
					$_energy_constant_flux_direct_layer_above = $_emmisivity_correction * $_energy_constant_above;
					

					$_energy_per_t_flux_direct_layer_below = $_emmisivity_correction * $_energy_per_t_below;
					$_energy_constant_flux_direct_layer_below = $_emmisivity_correction * $_energy_constant_below;
					$_energy_per_t_flux_reflection_layer_below = ( 1-$this->surfacealbedo ) * $_energy_per_t_flux_direct_layer_below;
					$_energy_constant_flux_reflection_layer_below = ( 1-$this->surfacealbedo ) * $_energy_constant_flux_direct_layer_below;
					
					/*
					echo "T direct: $_energy_per_t_flux_direct $_energy_constant_flux_direct\n";
					echo "T reflect: $_energy_per_t_flux_reflection $_energy_constant_flux_reflection\n";
					echo "Em Correction: $_emmisivity_correction\n";

					echo "T above direct: $_energy_per_t_flux_direct_layer_above $_energy_constant_flux_direct_layer_above\n";
					
					echo "T below direct: $_energy_per_t_flux_direct_layer_below $_energy_constant_flux_direct_layer_below\n";
					echo "T below reflect: $_energy_per_t_flux_reflection_layer_below $_energy_constant_flux_reflection_layer_below\n";
					*/
					
					//up or down, flux up is positive
					if ( $layer < $n )
					{
						//energy going upwards

						

						//diminish the energy for every layer crossed

						for( $i=0; $i < $n; $i++ )
						{
							if ( $i > $layer )
							{
								$_energy_per_t_flux_direct = $_energy_per_t_flux_direct * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_per_t_flux_reflection = $_energy_per_t_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);

								$_energy_constant_flux_direct = $_energy_constant_flux_direct * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_reflection = $_energy_constant_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);


								$_energy_per_t_flux_direct_layer_above = $_energy_per_t_flux_direct_layer_above * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_direct_layer_above = $_energy_constant_flux_direct_layer_above * (1 - $this->layers[$i]->absSpectrum[$k]['a']);	

								$_energy_per_t_flux_reflection_layer_below = $_energy_per_t_flux_reflection_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_reflection_layer_below = $_energy_constant_flux_reflection_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);

							}
							else if( $i ==  $layer )
							{
								//the reflected energy travels through itself once
								$_energy_per_t_flux_reflection = $_energy_per_t_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_reflection = $_energy_constant_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);


								$_energy_per_t_flux_reflection_layer_below = $_energy_per_t_flux_reflection_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_reflection_layer_below = $_energy_constant_flux_reflection_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
							}
							else
							{
								//the reflected energy travels through each layer below $layer twice
								$_energy_per_t_flux_reflection = $_energy_per_t_flux_reflection * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
								$_energy_constant_flux_reflection = $_energy_constant_flux_reflection * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );

								$_energy_per_t_flux_reflection_layer_below = $_energy_per_t_flux_reflection_layer_below * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
								$_energy_constant_flux_reflection_layer_below = $_energy_constant_flux_reflection_layer_below * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
							}
						}
						
						//energy going upwards - we are interested in the emmisivity correction on the above side

						$_total_surface_radiation_above['constant'] += $_energy_constant_flux_direct_layer_above; 
						$_total_surface_radiation_above['perdeltat'] += $_energy_per_t_flux_direct_layer_above;



					}
					else   //( $layer >= $n )
					{
						//we reverse the sign of the direct energy, as is coming down
						$_energy_per_t_flux_direct = 0 - $_energy_per_t_flux_direct;
						$_energy_constant_flux_direct = 0 - $_energy_constant_flux_direct;

						$_energy_per_t_flux_direct_layer_below = 0 - $_energy_per_t_flux_direct_layer_below;
						$_energy_constant_flux_direct_layer_below = 0 - $_energy_constant_flux_direct_layer_below;

						for( $i=$layer-1; $i >= 0; $i-- )
						{
							if ( $i >= $n )
							{
								$_energy_per_t_flux_direct = $_energy_per_t_flux_direct * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_per_t_flux_reflection = $_energy_per_t_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);

								$_energy_constant_flux_direct = $_energy_constant_flux_direct * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_reflection = $_energy_constant_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);

								$_energy_per_t_flux_direct_layer_below = $_energy_per_t_flux_direct_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_direct_layer_below = $_energy_constant_flux_direct_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);	

								$_energy_per_t_flux_reflection_layer_below = $_energy_per_t_flux_reflection_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_constant_flux_reflection_layer_below = $_energy_constant_flux_reflection_layer_below * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
							}
							else
							{
								//the reflected energy travels through each layer below $layer twice
								$_energy_per_t_flux_reflection = $_energy_per_t_flux_reflection * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
								$_energy_constant_flux_reflection = $_energy_constant_flux_reflection * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );

								$_energy_per_t_flux_reflection_layer_below = $_energy_per_t_flux_reflection_layer_below * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
								$_energy_constant_flux_reflection_layer_below = $_energy_constant_flux_reflection_layer_below * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
							}
						}
						
						//energy from layer above, we are now interested in emmisivity correction the downward side
						$_total_surface_radiation_below['constant'] +=  $_energy_constant_flux_direct_layer_below + $_energy_constant_flux_reflection_layer_below;
						$_total_surface_radiation_below['perdeltat'] +=  $_energy_per_t_flux_direct_layer_below + $_energy_per_t_flux_reflection_layer_below;


						if ( $createSpectrum )
						{
							$a_spectrum[$k]['E'] += $_energy_constant_flux_direct_layer_below + $_energy_constant_flux_reflection_layer_below;
							$a_spectrum[$k]['E'] += ($_energy_per_t_flux_direct_layer_below + $_energy_per_t_flux_reflection_layer_below) * $this->layers[$layer-1]->T; 

						}
					}

					$_total_surface_radiation['constant'] +=  $_energy_constant_flux_direct + $_energy_constant_flux_reflection;
					$_total_surface_radiation['perdeltat'] +=  $_energy_per_t_flux_direct + $_energy_per_t_flux_reflection;

					

					//echo "total_surface_radiation: ".$_total_surface_radiation['perdeltat']." ".$_total_surface_radiation['constant']."\n";
					//echo "total_surface_radiation above: ".$_total_surface_radiation_above['perdeltat']." ".$_total_surface_radiation_above['constant']."\n";
					//echo "total_surface_radiation below: ".$_total_surface_radiation_below['perdeltat']." ".$_total_surface_radiation_below['constant']."\n";

					//Calculate cloud reflections

					if ( $this->cloudParameters )
					{
						if ( $n <= $this->cloudParameters['base'] )
						{

							//we are under the cloud
							
							$_above_n_cloud_att = $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];
							$_below_n_surface_att = $this->surfacealbedo;
							$_above_layer_cloud_att = $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];

							$_ratio = $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'] * $this->surfacealbedo;

							for( $i=0; $i<$this->cloudParameters['base']; $i++ )
							{
								$_ratio = $_ratio * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );

								if ( $i >= $n )
								{
									$_above_n_cloud_att = $_above_n_cloud_att * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2);
								}
								else
								{	
									$_below_n_surface_att = $_below_n_surface_att * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2);
								}

								if ( $i > $layer )
								{
									$_above_layer_cloud_att = $_above_layer_cloud_att * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2);
								}
								else if ( $i == $layer )
								{
									//for layers, it goes thru once only
									$_above_layer_cloud_att = $_above_layer_cloud_att * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								}								
 
							}

							if ( $layer < $n )
							{
								//direct flux reflected down once then series addition
								//direct flux reflected from cloud then surface then series addition								
								//reflected flux refeclected from cloud above, then series addition
								//refelcted flux reflected from cloud then surface, then series addition


								//So we can simplify this as:

								$_total_surface_radiation['constant'] +=  ( $_below_n_surface_att - 1) * $_energy_constant_flux_direct * $_above_n_cloud_att * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation['perdeltat'] += ( $_below_n_surface_att - 1) * $_energy_per_t_flux_direct * $_above_n_cloud_att * (  1 / (1 - $_ratio)  ); 
								$_total_surface_radiation['constant'] +=  ( $_below_n_surface_att - 1) * $_energy_constant_flux_reflection * $_above_n_cloud_att * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation['perdeltat'] += ( $_below_n_surface_att - 1) * $_energy_per_t_flux_reflection * $_above_n_cloud_att * (  1 / (1 - $_ratio)  );

								//applied to the emmisivity corrections, the radiation first upwards, so applies to the above								

								$_total_surface_radiation_above['constant'] +=  ( $_below_n_surface_att - 1) * $_energy_constant_flux_direct_layer_above * $_above_n_cloud_att * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation_above['perdeltat'] += ( $_below_n_surface_att - 1) * $_energy_per_t_flux_direct_layer_above * $_above_n_cloud_att * (  1 / (1 - $_ratio)  ); 

								//applied to the emmisivity corrections, the radiation first downwards, so applied to the below

								$_total_surface_radiation_below['constant'] +=  ( $_below_n_surface_att - 1) * $_energy_constant_flux_reflection_layer_below * $_above_n_cloud_att * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation_below['perdeltat'] += ( $_below_n_surface_att - 1) * $_energy_per_t_flux_reflection_layer_below * $_above_n_cloud_att * (  1 / (1 - $_ratio)  );

							

							}
							else // $layer >= $n
							{
								if ( $layer < $this->cloudParameters['base'] )
								{
									//we have a downward reflection, but only for layers below the cloud
									//it is the direct amount attenuated by the layers above^2 and cloud reflection
									//the direct flux is negative, and so is the reflection, so keep it as an addition (to keep it negative) 									

									$_total_surface_radiation['constant'] +=  $_above_layer_cloud_att * $_energy_constant_flux_direct * (  1 / (1 - $_ratio)  );
									$_total_surface_radiation['perdeltat'] += $_above_layer_cloud_att * $_energy_per_t_flux_direct * (  1 / (1 - $_ratio)  );

									//the upward amount will be the surface reflection attenuated by all layers below n^2 $_1st_up_ratio
									//the value is positive, so reverse the negative value
									
									$_total_surface_radiation['constant'] -=  $_above_layer_cloud_att * $_below_n_surface_att * $_energy_constant_flux_direct * (  1 / (1 - $_ratio)  );
									$_total_surface_radiation['perdeltat'] -= $_above_layer_cloud_att * $_below_n_surface_att *$_energy_per_t_flux_direct * (  1 / (1 - $_ratio)  );

									//applied to the emmisivity correctors
									//this is upward to begin with, so apply to above

									$_total_surface_radiation_above['constant'] += 	$_above_layer_cloud_att * $_energy_constant_flux_direct_layer_above * (  1 / (1 - $_ratio)  );									
									$_total_surface_radiation_above['perdeltat'] += $_above_layer_cloud_att * $_energy_per_t_flux_direct_layer_above * (  1 / (1 - $_ratio)  );

									//and again for the  surface re-reflections

									$_total_surface_radiation_above['constant'] -=  $_above_layer_cloud_att * $_below_n_surface_att * $_energy_constant_flux_direct_layer_above * (  1 / (1 - $_ratio)  );
									$_total_surface_radiation_above['perdeltat'] -= $_above_layer_cloud_att * $_below_n_surface_att * $_energy_per_t_flux_direct_layer_above * (  1 / (1 - $_ratio)  );


								}

								//now apply reflections to the surface reflections

								//the first downward amount, make it negative (surface reflections are positive)
								//attenuated by the layers above n^2								

								$_total_surface_radiation['constant'] -=   $_above_n_cloud_att * $_energy_constant_flux_reflection * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation['perdeltat'] -=  $_above_n_cloud_att * $_energy_per_t_flux_reflection * (  1 / (1 - $_ratio)  );

								//the primary surface reflection is dealt with, but the second reflections attenuated by an entire reflecton cycle $_ratio

								$_total_surface_radiation['constant'] +=   $_ratio * $_energy_constant_flux_reflection * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation['perdeltat'] +=  $_ratio * $_energy_per_t_flux_reflection * (  1 / (1 - $_ratio)  );

									
								//the same again for the emmisivity correctors
								$_total_surface_radiation_below['constant'] -=  $_above_n_cloud_att * $_energy_constant_flux_reflection_layer_below * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation_below['perdeltat'] -=  $_above_n_cloud_att * $_energy_per_t_flux_direct_layer_below * (  1 / (1 - $_ratio)  );
								

								//and the second reflections upwards
								$_total_surface_radiation_below['constant'] +=   $_ratio * $_energy_constant_flux_reflection_layer_below * (  1 / (1 - $_ratio)  );
								$_total_surface_radiation_below['perdeltat'] +=  $_ratio * $_energy_per_t_flux_direct_layer_below * (  1 / (1 - $_ratio)  );

							}					

								
						}
						else if ( $n > $this->cloudParameters['top'] )
						{
							//above the cloud level, there is only one reflection	
							$_below_both = $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'];
							$_surface_bounces = $this->cloudParameters['longwave']['albedo'] * $this->cloudParameters['cover'] * $this->surfacealbedo;

							for( $i=0; $i < $this->cloudParameters['base']; $i++ )
							{
								$_surface_bounces = $_surface_bounces * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2);			
							}
							
							if ( $layer > $this->cloudParameters['top'] )
							{
								//attenuation ratio is the square of the layers below either $n or $layer whichever is lowest	
								for( $i = $this->cloudParameters['top'] +1; $i < min( $n, ($layer-1) ); $i++ )
								{
									$_below_both = $_below_both * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2);
								}

								if ( ($layer < $n)  ) //check the signs
								{
									$_total_surface_radiation['constant'] +=  $_energy_constant_flux_direct  * $_below_both;
									$_total_surface_radiation['perdeltat'] +=  $_energy_per_t_flux_direct  * $_below_both; 

									//emmisivty corrections, apply to below layer
									
									$_total_surface_radiation_below['constant'] +=  $_energy_constant_flux_direct_layer_below  * $_below_both;
									$_total_surface_radiation_below['perdeltat'] +=  $_energy_per_t_flux_direct_layer_below  * $_below_both; 
								}
								else 
								{
									//we need to attenuate through $layer as well once for this situation
									$_below_both = $_below_both * (1 - $this->layers[$layer]->absSpectrum[$k]['a']);

									//remember direct flux is negative! make it positive

									$_total_surface_radiation['constant'] -=  $_energy_constant_flux_direct * $_below_both;
									$_total_surface_radiation['perdeltat'] -=  $_energy_per_t_flux_direct * $_below_both; 

									//emmisivty corrections, apply to below layer
									
									$_total_surface_radiation_below['constant'] -=  $_energy_constant_flux_direct_layer_below  * $_below_both;
									$_total_surface_radiation_below['perdeltat'] -=  $_energy_per_t_flux_direct_layer_below  * $_below_both;
								}
							}
							
							//now reflections of surface reflections
							//these apply for all circumstances

							$_total_surface_radiation['constant'] +=  $_energy_constant_flux_reflection * $_surface_bounces * ( 1 / 1 - $_surface_bounces );
							$_total_surface_radiation['perdeltat'] +=  $_energy_per_t_flux_reflection * $_surface_bounces * ( 1 / 1 - $_surface_bounces );
		
						}

					}


					
				}


				$a_mattrix['flux'][ $layer+1 ] = $a_mattrix['flux'][ $layer+1 ] + $_total_surface_radiation['perdeltat'];

				
				
				$this->a_vector_of_linear_fit[$n] = $this->a_vector_of_linear_fit[$n] - $_total_surface_radiation['constant'];

				if( $this->verbose ){
					echo "Layer ".$layer." flux coefficient: (".$_total_surface_radiation['perdeltat']." * T) + ".$_total_surface_radiation['constant']."\n";
				}
				else
					$this->output( "Layer ".$layer." flux coefficient: (".$_total_surface_radiation['perdeltat']." * T) + ".$_total_surface_radiation['constant']."\n" );

				

				if ( $layer < $this->noLayers-1 )
				{
					$a_mattrix['flux'][ $layer+2 ] = $a_mattrix['flux'][ $layer+2 ] + $_total_surface_radiation_above['perdeltat'];
					$this->a_vector_of_linear_fit[$n] = $this->a_vector_of_linear_fit[$n] - $_total_surface_radiation_above['constant'];

					if( $this->verbose ){
						echo "Above Layer EMC ".($layer+1)." flux coefficient: (".$_total_surface_radiation_above['perdeltat']." * T) + ".$_total_surface_radiation_above['constant']."\n";
					}
					else
						$this->output( "Above Layer EMC ".($layer+1)." flux coefficient: (".$_total_surface_radiation_above['perdeltat']." * T) + ".$_total_surface_radiation_above['constant']."\n" );
				}

				
				$a_mattrix['flux'][ $layer ] = $a_mattrix['flux'][ $layer ] + $_total_surface_radiation_below['perdeltat'];
				$this->a_vector_of_linear_fit[$n] = $this->a_vector_of_linear_fit[$n] - $_total_surface_radiation_below['constant'];

				if( $this->verbose ){
					echo "Below Layer EMC ".($layer-1)." flux coefficient: (".$_total_surface_radiation_below['perdeltat']." * T) + ".$_total_surface_radiation_below['constant']."\n";
				}
				else
					$this->output( "Below Layer EMC ".($layer-1)." flux coefficient: (".$_total_surface_radiation_below['perdeltat']." * T) + ".$_total_surface_radiation_below['constant']."\n" );
	
			
				$this->outputStatus(  ($n/($this->noLayers+1)) + ( ($layer+1)/ ( pow(($this->noLayers+1),2) ) ), 'Deducing Flux Equations' );				

		}

		if( $this->verbose )
			echo "Layer ".$n." vector entry: ".$this->a_vector_of_linear_fit[$n]."\n";
		else
			$this->output( "Layer ".$n." vector entry: ".$this->a_vector_of_linear_fit[$n]."\n" );	

		//calculate the convection

		if ( $this->add_dry_convection && (  $n <  ($this->noLayers ) ) )		//can't convect into space!
		{

			//the mathematics as follows:  
			//C(T1,T2) = C(t1,t2) +  (T1-t1). dC/dt1 + (T2-t2) . dC/dt1	
			//         = C(t1,t2) + T1.dC/dt1 - t1.dC/dt1 + T2.dC/dt1 - t2.dCdt2
			//	   = C(t1,t2) - t1.dC/dt1 - t2.dCdt2   +  T1.dC/dt1 + T2.dC/dt1	
			// where t1 and t2 are the values we have, and T1 and T2 are the values to be solved	

			if ( $n > 0 )
			{		
				
				$Pressure = (($this->noLayers - $n - 0.5)/$this->noLayers) * TOTAL_ATMOSPHERE;
				$deltaT = $this->layers[$n-1]->T - $this->layers[$n]->T;

				$_convection_constant = $this->getDryConvection( $this->layers[$n-1]->T, $this->layers[$n]->T, $Pressure );
				$_convection_perdeltat1 = $this->getDryConvection_dEddT( $this->layers[$n-1]->T, $this->layers[$n]->T, $Pressure, $derivateToT=1  );
				$_convection_perdeltat2 = $this->getDryConvection_dEddT( $this->layers[$n-1]->T, $this->layers[$n]->T, $Pressure, $derivateToT=2  );

				$_convection_constant_addition_to_solution_vector = $_convection_constant - ($_convection_perdeltat1 * $this->layers[$n-1]->T ) - ($_convection_perdeltat2 * $this->layers[$n]->T );
				
				$Ad_lapserate = $this->getDryAdiabaticLapseRate( ($this->layers[$n-1]->T + $this->layers[$n]->T)/2, $Pressure );
			}		
			else
			{
				$Pressure = (($this->noLayers - 0.5)/$this->noLayers) * TOTAL_ATMOSPHERE;
				$deltaT = $this->layers['s']->T - $this->layers[0]->T;

				//note, scale is half as it is between surface T to mid 0-100 hPa.

				$_convection_constant = $this->getDryConvection( $this->layers['s']->T, $this->layers[0]->T, $Pressure, $scale=0.5 );
				$_convection_perdeltat1 = $this->getDryConvection_dEddT( $this->layers['s']->T, $this->layers[0]->T, $Pressure, $derivateToT=1, $scale=0.5 );
				$_convection_perdeltat2 = $this->getDryConvection_dEddT( $this->layers['s']->T, $this->layers[0]->T, $Pressure, $derivateToT=2, $scale=0.5 );

				$_convection_constant_addition_to_solution_vector = $_convection_constant - ($_convection_perdeltat1 * $this->layers['s']->T ) - ($_convection_perdeltat2 * $this->layers[0]->T );
				
				$Ad_lapserate = $this->getDryAdiabaticLapseRate( ($this->layers['s']->T + $this->layers[0]->T)/2, $Pressure );

			}

					
			$a_mattrix['flux'][$n+1] += $_convection_perdeltat2;
			$a_mattrix['flux'][$n] += $_convection_perdeltat1;
			
			$this->a_vector_of_linear_fit[$n] -= $_convection_constant_addition_to_solution_vector;

			if( $this->verbose )
				echo "Layer ".$n." convection(dry): (dT1dT:".$_convection_perdeltat1.",dT2/dT:".$_convection_perdeltat2." * [T2-T1=".$deltaT."/Ld=".$Ad_lapserate."]) + ".$_convection_constant_addition_to_solution_vector."\n";
			else
				$this->output( "Layer ".$n." convection(dry): (dT1dT:".$_convection_perdeltat1.",dT2/dT:".$_convection_perdeltat2." * [T2-T1=".$deltaT."/Ld=".$Ad_lapserate."]) + ".$_convection_constant_addition_to_solution_vector."\n" );

		}

		if ( $this->add_wet_convection && (  $n <  ($this->noLayers ) ) )		//can't convect into space!
		{

			//the mathematics as follows:  
			//C(T1,T2) = C(t1,t2) +  (T1-t1). dC/dt1 + (T2-t2) . dC/dt1	
			//         = C(t1,t2) + T1.dC/dt1 - t1.dC/dt1 + T2.dC/dt1 - t2.dCdt2
			//	   = C(t1,t2) - t1.dC/dt1 - t2.dCdt2   +  T1.dC/dt1 + T2.dC/dt1	
			// where t1 and t2 are the values we have, and T1 and T2 are the values to be solved	

			if ( $n > 0 )
			{		
				
				$Pressure = (($this->noLayers - $n - 0.5)/$this->noLayers) * TOTAL_ATMOSPHERE;
				$deltaT = $this->layers[$n-1]->T - $this->layers[$n]->T;

				$_convection_constant = $this->getWetConvection( $this->layers[$n-1]->T, $this->layers[$n]->T, $Pressure );
				$_convection_perdeltat1 = $this->getWetConvection_dEddT( $this->layers[$n-1]->T, $this->layers[$n]->T, $Pressure, $derivateToT=1  );
				$_convection_perdeltat2 = $this->getWetConvection_dEddT( $this->layers[$n-1]->T, $this->layers[$n]->T, $Pressure, $derivateToT=2  );

				$_convection_constant_addition_to_solution_vector = $_convection_constant - ($_convection_perdeltat1 * $this->layers[$n-1]->T ) - ($_convection_perdeltat2 * $this->layers[$n]->T );
				
				$Ad_lapserate = $this->getWetAdiabaticLapseRate( ($this->layers[$n-1]->T + $this->layers[$n]->T)/2, $Pressure );
			}		
			else
			{
				$Pressure = (($this->noLayers - 0.5)/$this->noLayers) * TOTAL_ATMOSPHERE;
				$deltaT = $this->layers['s']->T - $this->layers[0]->T;

				$_convection_constant = $this->getWetConvection( $this->layers['s']->T, $this->layers[0]->T, $Pressure, $scale=0.5 );
				$_convection_perdeltat1 = $this->getWetConvection_dEddT( $this->layers['s']->T, $this->layers[0]->T, $Pressure, $derivateToT=1, $scale=0.5 );
				$_convection_perdeltat2 = $this->getWetConvection_dEddT( $this->layers['s']->T, $this->layers[0]->T, $Pressure, $derivateToT=2, $scale=0.5 );

				$_convection_constant_addition_to_solution_vector = $_convection_constant - ($_convection_perdeltat1 * $this->layers['s']->T ) - ($_convection_perdeltat2 * $this->layers[0]->T );
				
				$Ad_lapserate = $this->getWetAdiabaticLapseRate( ($this->layers['s']->T + $this->layers[0]->T)/2, $Pressure );

			}

					
			$a_mattrix['flux'][$n+1] += $_convection_perdeltat2;
			$a_mattrix['flux'][$n] += $_convection_perdeltat1;
			
			$this->a_vector_of_linear_fit[$n] -= $_convection_constant_addition_to_solution_vector;

			if( $this->verbose )
				echo "Layer ".$n." convection(wet): (dT1dT:".$_convection_perdeltat1.",dT2/dT:".$_convection_perdeltat2." * [T2-T1=".$deltaT."/Ld=".$Ad_lapserate."]) + ".$_convection_constant_addition_to_solution_vector."\n";
			else		
				$this->output( "Layer ".$n." convection(wet): (dT1dT:".$_convection_perdeltat1.",dT2/dT:".$_convection_perdeltat2." * [T2-T1=".$deltaT."/Ld=".$Ad_lapserate."]) + ".$_convection_constant_addition_to_solution_vector."\n"  );
		}

		return 	array( 'deltat' => $a_mattrix['flux'], 'const' => $this->a_vector_of_linear_fit[$n] );
	}

	
	/*
	function deduceEquationUsingT4( $n )
	{
		//inputs from surface
		//all radiation going up
		
		//surface variable
		//$this->temperatureLayersToBeSolved[0];

		if( $this->verbose )
			echo "Deducing flux inputs for layer $n \n";

		if ( $this->add_dry_convection && $this->verbose )
			echo "Warning: this method (deduceEquationUsingT4) does not work with convection, convection will be ignored\n";
		
		$_total_surface_raditaion_per_t4 = 0;
		$total_layer_energy_in = 0;

		//How much of the surface raditation makes it to the nth layer



		for( $k=0; $k<sizeof($this->surfaceSpectrumInfraRed); $k++ )
		{	

			if ( $this->layers['s']->T )
			{
				$_energy_per_t4 = ($this->surfacealbedo * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers['s']->T )  * $this->surfaceSpectrumInfraRed[$k]['i'] ) / (pow( $this->layers['s']->T, 4 ));
			}
			else
			{

				//$energy +=  $this->surfaceSpectrumInfraRed[$k]['e'] * $this->surfaceSpectrumInfraRed[$k]['i'];
				$_energy_per_t4 = ($this->surfaceSpectrumInfraRed[$k]['e'] * $this->surfaceSpectrumInfraRed[$k]['i'] * SIGMA * $this->surfacealbedo) / $this->Es;
			}

			//now absorb this energy for each layer in between

			for( $i=0; $i<$n; $i++ )
			{
				$_energy_per_t4 = $_energy_per_t4 * (1 - $this->layers[$i]->absSpectrum[$k]['a']);

			}

			$_total_surface_raditaion_per_t4 +=  $_energy_per_t4;

		}		
		

		$a_mattrix['flux'][ 0 ] = $_total_surface_raditaion_per_t4;

		if( $this->verbose )
			echo "Layer 0 flux coefficient: $_total_surface_raditaion_per_t4 \n";

		//now do one for each layer


		for( $layer = 0; $layer< $this->noLayers; $layer++ )
		{
			
				//calculate radiation from layer reaching n
			

				$_total_flux_from_layer_to_n = 0;
				
				for( $k=0; $k<sizeof($this->surfaceSpectrumInfraRed); $k++ )
				{
					//the portion of energy from layer

					if ( !($_energy_per_t4 = $this->_energy_cache[ $layer ][ $k ]) )
					{
						if ( $this->layers[$layer]->T )
						{
							$_energy_per_t4 = ($this->layers[ $layer ]->absSpectrum[$k]['a'] * $this->getEFromPlanksEquations( $this->surfaceSpectrumInfraRed[$k]['y'], $this->layers[$layer]->T  ) * $this->surfaceSpectrumInfraRed[$k]['i'] ) / (pow( $this->layers[$layer]->T, 4 ));
						}
						else
						{

							$_energy_per_t4 = $this->layers[ $layer ]->absSpectrum[$k]['a'] *  SIGMA * ($this->surfaceSpectrumInfraRed[$k]['e'] * $this->surfaceSpectrumInfraRed[$k]['i'] / ($this->Es * $this->surfacealbedo ));
							
						}

						$this->_energy_cache[ $layer ][ $k ] = $_energy_per_t4;
					}

					
					//from each layer there is a direct energy and the energy reflected by the surface
					$_energy_per_t4_flux_direct = $_energy_per_t4;
					$_energy_per_t4_flux_reflection = $_energy_per_t4 * (1-$this->surfacealbedo);

					
					//up or down, flux up is positive
					if ( $layer < $n )
					{
						//diminish the energy for every layer crossed

						for( $i=0; $i < $n; $i++ )
						{
							if ( $i > $layer )
							{
								$_energy_per_t4_flux_direct = $_energy_per_t4_flux_direct * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_per_t4_flux_reflection = $_energy_per_t4_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
							}
							else if( $i ==  $layer )
							{
								//the reflected energy travels through itself once
								$_energy_per_t4_flux_reflection = $_energy_per_t4_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
							}
							else
							{
								//the reflected energy travels through each layer below $layer twice
								$_energy_per_t4_flux_reflection = $_energy_per_t4_flux_reflection * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
							}
						}
					}
					else   //( $layer >= $n )
					{
						//we reverse the sign of the direct energy, as is coming down
						$_energy_per_t4_flux_direct = 0 - $_energy_per_t4_flux_direct;

						for( $i=$layer-1; $i >= 0; $i-- )
						{
							if ( $i >= $n )
							{
								$_energy_per_t4_flux_direct = $_energy_per_t4_flux_direct * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
								$_energy_per_t4_flux_reflection = $_energy_per_t4_flux_reflection * (1 - $this->layers[$i]->absSpectrum[$k]['a']);
							}
							else
							{
								//the reflected energy travels through each layer below $layer twice
								$_energy_per_t4_flux_reflection = $_energy_per_t4_flux_reflection * pow( (1 - $this->layers[$i]->absSpectrum[$k]['a']), 2 );
							}
						}
					}

					$_total_flux_from_layer_to_n +=  $_energy_per_t4_flux_direct + $_energy_per_t4_flux_reflection;

					
				}


				$a_mattrix['flux'][ $layer+1 ] = $_total_flux_from_layer_to_n;

				if( $this->verbose )
					echo "Layer ".($layer+1)." flux coefficient: ".$_total_flux_from_layer_to_n."\n";
				

								

				//energy balance equation
			
		}

		$this->a_vector_of_linear_fit[$n] = SURFACE_SOLAR_FLUX;

		return 	array( 'pert4' => $a_mattrix['flux'], 'const' => $this->a_vector_of_linear_fit[$n] );
	}
	*/


	function create_Mattrix( $n=NULL )
	{
		if ( $n !== NULL )
		{
			if ( $n === 0 )
				unset( $this->_energy_cache );

			$a = $this->deduceEquationUsingLinear( $n );
			$this->a_mattrix_of_coefficients[$n] = $a['deltat'];
		}
		else
		{

			unset( $this->_energy_cache );

			for( $i=0; $i<=$this->noLayers; $i++ )
			{
				//if ( $this->method == "t4coeffs" )
				//{
				//	$a_ = $this->deduceEquationUsingT4( $i );
				//	$a_mattrix[] = $a_['pert4'];
				//}
				//else
				//{
					$a_ = $this->deduceEquationUsingLinear( $i );
					$a_mattrix[] = $a_['deltat'];
				//}
			
			}

			$this->a_mattrix_of_coefficients = &$a_mattrix;
		}

	}

	

	function solveMattrix()
	{
		$this->outputStatus( 0, 'Solving Mattrix' );

		if ( !is_array( $this->a_vector_of_linear_fit ) )
		{
			$this->output( "Solution vector has not been created - has deduceEquationUsingCalculus been run?\n");
			die('Solution vector has not been created - has deduceEquationUsingCalculus been run?');
		}

		if ( !is_array( $this->a_mattrix_of_coefficients ) )
		{
			$this->output( "Coefficients matrix has not been created - has deduceEquationUsingCalculus been run?\n");
			die( 'Coefficients matrix has not been created - has deduceEquationUsingCalculus been run?' );	
		}

		$MM = new Math_Matrix( $a = $this->a_mattrix_of_coefficients );
		$MV = new Math_Vector( $a = $this->a_vector_of_linear_fit );
		$V = Math_Matrix::solveEC( $MM,  $MV );

		$result = $V->getData();

		$this->result = $result;

		for( $j=0; $j<sizeof( $result ); $j++ )
		{
			if ( $j == 0 )
			{
				$this->layers['s'] = new Layer();

				if ( $this->method == "t4coeffs" )
				{ 
					$this->layers['s']->T = pow($result[$j], 1/4 );
					$_result['s'] = pow($result[$j], 1/4 );

				}	
				else
				{
					$this->layers['s']->T = $result[$j];
					$_result['s'] = $result[$j];
				}
			}
			else{

				if ( $this->method == "t4coeffs" )
				{ 
					$this->layers[$j-1]->T = pow( $result[$j], 1/4 );
					$_result[$j-1] = pow( $result[$j], 1/4 );

				}
				else
				{
					$this->layers[$j-1]->T = $result[$j];
					$_result[$j-1] = $result[$j];
				}
			}

			if( $this->verbose )
			{
				if ( $j == 0 )
					echo "Layer (s) : ". $_result['s'] ."\n";
				else
					echo "Layer ".$j." : ". $_result[$j-1] ."\n";
			}
			else
			{
				if ( $j == 0 )
					$this->output( "Layer (s) : ". $_result['s'] ."\n");
				else
					$this->output( "Layer ".$j." : ". $_result[$j-1] ."\n" );
			}
		}

		if ( is_array( $this->resultSet ) )
		{
			$_index = sizeof( $this->resultSet ) - 1;
			$_dev = 0;
			$_dir = 0;

			for( $i=0; $i<=$this->noLayers; $i ++ )
			{
				$_dev += pow( ($this->resultSet[ $_index ]['values'][$i] - $result[$i]), 2 );
				$_dir += $this->resultSet[ $_index ]['values'][$i] - $result[$i];
			}

			$_dev = pow( ($_dev/($this->noLayers + 1)) , 0.5 );
			$_dir = $_dir / ($this->noLayers + 1);	
		}
		else
			$_dev = -1;

		$this->resultSet[$_index+1] = array( 'values'=>$result, 'deviation'=> $_dev, 'direction'=>$_dir );

		if( $this->verbose )
			echo 'Deviation: '.$_dev."\n";
		else
			$this->output( 'Deviation: '.$_dev."\n" );

		$this->outputStatus( 1, 'Solving Mattrix' );

		return  $_result;

	}

	function bounceCorrection()
	{
		if ( sizeof( $this->resultSet ) >= 2 )
		{
			$_index = sizeof( $this->resultSet ) - 1;
			
			if ( ($this->resultSet[$_index]['direction'] > 0) && ($this->resultSet[$_index-1]['direction'] < 0) )
				$bool_bounce=true;

			if ( ($this->resultSet[$_index]['direction'] < 0) && ($this->resultSet[$_index-1]['direction'] > 0) )
				$bool_bounce=true;

			if ( $bool_bounce )
			{
				if( $this->verbose )
					echo "Bounce Correction being calculated\n";
				else
					$this->output(  "Bounce Correction being calculated\n" );

				for( $k=0; $k<sizeof( $this->resultSet[$_index]['values']); $k++ )
				{
					$delta = $this->resultSet[$_index]['values'][$k] - $this->resultSet[$_index-1]['values'][$k]; 
					$weighting = $this->resultSet[$_index]['deviation'] / (  $this->resultSet[$_index]['deviation'] + $this->resultSet[$_index-1]['deviation']  );
					$corrected_value = $this->resultSet[$_index]['values'][$k] - ($weighting * $delta);

					if ( $k == 0 )
					{
						$this->layers['s']->T = $corrected_value;
					}
					else
					{
						$this->layers[$k-1]->T = $corrected_value;
					}
				}

			}		
		}
	}

	function getDeviation()
	{
		if ( is_array($this->resultSet) )
		{
			$_index = sizeof( $this->resultSet ) - 1;

			return $this->resultSet[ $_index ]['deviation'];
		}
		else
			return false;
	}

	function getDirection()
	{
		if ( is_array($this->resultSet) )
		{
			$_index = sizeof( $this->resultSet ) - 1;

			return $this->resultSet[ $_index ]['direction'];
		}
		else
			return false;
	}


	function getEFromPlanksEquations( $y, $T )
	{ 

		if ( $T == 0 )
			die('Temperature zero');

		if ( $y == 0 )
			die('Wavelength zero');

		$y = $y * pow( 10, -6 );

		

		//echo 'here: '.( (H_PLANCKS_CONSTANT * C_SPEED_OF_LIGHT)/ ($y * K_BOLTZMANN_CONSTANT * $T ) ) ."\n";
		//echo 'here: '.exp( (H_PLANCKS_CONSTANT * C_SPEED_OF_LIGHT)/ ($y * K_BOLTZMANN_CONSTANT * $T ) ) ."\n" ;
		//exit;

		return 2 * H_PLANCKS_CONSTANT * pow(C_SPEED_OF_LIGHT,2)  /  ( pow($y,5)  *  ( exp( (H_PLANCKS_CONSTANT * C_SPEED_OF_LIGHT)/($y * K_BOLTZMANN_CONSTANT * $T)  ) - 1) ) * pow(10,-6) * pi();

	}

	function getPartialDerivative_dEdT_FromPlanksEquations( $y, $T )
	{ 

		if ( $T == 0 )
			die('Temperature zero');

		if ( $y == 0 )
			die('Wavelength zero');

		$y = $y * pow( 10, -6 );

		
		$A_ = 2 * H_PLANCKS_CONSTANT * pow(C_SPEED_OF_LIGHT,2);
		$B_ = pow( $y, 5 );
		$C_ =  (H_PLANCKS_CONSTANT * C_SPEED_OF_LIGHT) / ($y * K_BOLTZMANN_CONSTANT);
		$U_ = exp( $C_ / $T ) - 1;

		
		return

		($A_ / ($B_ * pow( $U_, 2 )))

		*

		exp( $C_ / $T )

		*

		($C_ / (pow( $T, 2 )))

		*

		(pow(10,-6) * pi());		


	}


	function checkBalance( $n )
	{
		
		$_sum = 0;

		for( $i=0; $i<sizeof($this->result); $i++ )
		{
			$_sum += $this->a_mattrix_of_coefficients[$n][$i] * $this->result[$i];
		}

		if( $this->verbose )
			echo $this->a_vector_of_solar_constants[$n].' : '.$_sum;
		
		return $this->a_vector_of_solar_constants[$i] - $_sum;
	}

	function getDryConvection( $T1, $T2, $P, $scale=1 )
	{
		$_thickness = ($scale * TOTAL_ATMOSPHERE/$this->noLayers);

		$deltaT = ($T1 - $T2) / $_thickness;  //K * hPa-1
		$avgT = ($T1 + $T2)/2;

		$_air_density = ( $P * 100 ) / (R_SPECIFIC_GAS_CONSTANT * $avgT );
		$_dry_adiabatic_lp =  100 / (  $_air_density * C_SPECIFIC_HEAT_DRY_AIR );  //K * hPa-1


		if ( ($deltaT - $_dry_adiabatic_lp ) > 0 )
		{		
			return $this->coefficient_of_dry_convection * pow( ( $deltaT  - $_dry_adiabatic_lp  ), 2  );
		}
		else
		{
			return 0;
		}
	}

	
	function getDryConvection_dEddT( $T1, $T2, $P, $partialOf = 1, $scale=1 )
	{
		$_Mt = (  $scale * TOTAL_ATMOSPHERE/$this->noLayers );
		$_Mt2 = pow($_Mt, 2);

		$deltaT = ($T1 - $T2) / $_Mt;  //K * hPa-1
		$avgT = ($T1 + $T2)/2;


		$_dry_adiabatic_lp =  (R_SPECIFIC_GAS_CONSTANT * $avgT ) / (  $P  * C_SPECIFIC_HEAT_DRY_AIR );

		$_d_dry_adiabatic_lp_dT = R_SPECIFIC_GAS_CONSTANT / ( $P * C_SPECIFIC_HEAT_DRY_AIR * 2 ); 



		if ( ($deltaT - $_dry_adiabatic_lp ) > 0 )
		{	
			if ( $partialOf == 1 )
			{
				return
				2 * $this->coefficient_of_dry_convection *
				( ($T1/$_Mt2) - ($T2/$_Mt2) - ($_dry_adiabatic_lp/$_Mt) - (($T1*$_d_dry_adiabatic_lp_dT)/$_Mt) + (($T2 *$_d_dry_adiabatic_lp_dT)/$_Mt) + ($_dry_adiabatic_lp*$_d_dry_adiabatic_lp_dT) );
				
			}
			else if ( $partialOf == 2 )
			{

				return 
				2 * $this->coefficient_of_dry_convection *
				( ($T2/$_Mt2)  - ($T1/$_Mt2) + ($_dry_adiabatic_lp/$_Mt) - (($T1*$_d_dry_adiabatic_lp_dT)/$_Mt) + (($T2 *$_d_dry_adiabatic_lp_dT)/$_Mt)  + ($_dry_adiabatic_lp*$_d_dry_adiabatic_lp_dT) );
			}		
		}
		else
		{
			return 0;
		}
	}

	function getDryAdiabaticLapseRate( $T, $P )
	{

		$_air_density = ( $P * 100 ) / (R_SPECIFIC_GAS_CONSTANT * $T );
		$_dry_adiabatic_lp =  100 / (  $_air_density * C_SPECIFIC_HEAT_DRY_AIR );
		//$_dry_adiabatic_lp = $_dry_adiabatic_lp * (TOTAL_ATMOSPHERE/$this->noLayers);

		return $_dry_adiabatic_lp;
	}

	function getSaturationVapourPressure( $T )
	{
		//Arden Buck equation

		$T = $T - 273.15;  //yes, this equation is in Celsius
	
		return 6.1121 * exp(   (  18.678 - ( $T/234.5)  ) * (  $T / ( 257.14 + $T ) )    );
	}

	function getSaturationVapourPressuredWdT( $T )
	{
		//derivative of Arden Buck equation with respect to T
		$T = $T - 273.15;

		$U = (  18.678 - ($T/234.5)  ) * (  $T / ( 257.14 + $T ) );
		$dUdT = ( (18.678 * ( 257.14 + $T ) )  -  ( 18.678 * $T ) ) / (  pow(  (257.14 + $T), 2 )  )
			-
			(( (60299.33 + 234.5 * $T) * 2 * $T) - ( 234.5 * pow( $T, 2 )  ) ) / (pow( ( 60299.33 + 257.14 * $T ), 2   ) );
			
		return 6.1121 * exp( $U ) * $dUdT;

	}	

	function getWetAdiabaticLapseRate( $T, $P )
	{
		

		$P = $P * 100;  //change to pascals from hPa

		$_air_density = $P / (R_SPECIFIC_GAS_CONSTANT * $T);

		//echo "Air Density: ".$_air_density."\n";
		//echo "Pw : ".Model::getSaturationVapourPressuredWdT( $T )."\n";		
				
		//echo "Ld(p): ".( 1 / (  $_air_density * C_SPECIFIC_HEAT_DRY_AIR ) )."\n";
		//echo "Ld(h): ".( G_ACCELERATION / (  C_SPECIFIC_HEAT_DRY_AIR ) )."\n";

		return
		100 
		/ 
		($_air_density * ( C_SPECIFIC_HEAT_DRY_AIR + (HEAT_OF_VAPORISATION * RATIO_OF_MASS_VAPOUR_TO_AIR * 100 * Model::getSaturationVapourPressuredWdT( $T ) / $P )  ));
                        

		// K hPa-1
	}

	function getWetConvection( $T1, $T2, $P, $scale=1 )
	{
		$_Mt = ($scale * TOTAL_ATMOSPHERE/$this->noLayers);

		$deltaT = ($T1 - $T2)/$_Mt;
		$avgT = ($T1 + $T2)/2;

		$_wet_adiabatic_lp = Model::getWetAdiabaticLapseRate( $avgT, $P );


		if ( ($deltaT - $_wet_adiabatic_lp ) > 0 )
		{		
			return $this->coefficient_of_wet_convection  * pow( ( $deltaT  - $_wet_adiabatic_lp  ), 2  );
		}
		else
		{
			return 0;
		}
	}

	function getWetConvection_dEddT( $T1, $T2, $P, $partialOf = 1, $scale=1 )
	{
		define( 'SMALL_INC', 0.0001 );
				
		//yes, I could derivate this, which is also dependant on getWetAdiabaticLapseRate, which in turn is dependant on getSaturationVapourPressuredWdT
		//But I am happy that I have proven to myself that I remember the Chain Rule, and Product rule and Quotient rule
		//So this time I am using:

		//The lazymans calculus!

		if ( $partialOf == 1 )
		{	
			return ($this->getWetConvection( $T1+SMALL_INC, $T2, $P, $scale ) - $this->getWetConvection( $T1, $T2, $P, $scale ) ) / SMALL_INC;

		}
		else if( $partialOf == 2 )
		{
			return ($this->getWetConvection( $T1, $T2+SMALL_INC, $P, $scale ) - $this->getWetConvection( $T1, $T2, $P, $scale ))/SMALL_INC;
		}
	}

	function setClouds( $a )
	{
		$this->cloudParameters = $a;
	}


	function getHighEmmissivityCorrection( $a, $calculate=false )
	{


		if ( $calculate )
		{
			define( "LARGE_NUMBER", 1000000 );

			if ( $this->verbose )
				echo "Calculating emmisivity correction for $a : this may take some time\n";
			else
				$this->output( "Calculating emmisivity correction for $a : this may take some time\n" );

			$_sum = 0;

			
			for( $j=0; $j<LARGE_NUMBER; $j++ )
			{
				$_sum += (pow( (1-$a), $j/LARGE_NUMBER ) - pow( ( 1 - $a ), (1+$j)/LARGE_NUMBER ))
					*
					( ( (LARGE_NUMBER / 2) - $j ) / LARGE_NUMBER  );
			}
			

			/*
			for( $j=0; $j<LARGE_NUMBER; $j++ )
			{

			$_sum += (($j/LARGE_NUMBER) - 0.5) * 
				(1 - pow( ( 1 - $a ),(1/LARGE_NUMBER) ) ) * 
				( pow( ( 1 - $a ), (1 - $j/LARGE_NUMBER) ) );

			}
			*/	
		
			if ( $this->verbose )
				echo "value: $_sum\n";
			else
				$this->output( "value: $_sum\n" );
	
			return $_sum;
				
		}
		else
		{
			if ( $a == 0 )
			{
				return 0;
			}
			else if ( $a == 1 )
			{
				return 0.5;
				
			}
			else
			{	
				$_index =  ceil( 0 - log( 1 - $a, 10 ) ) - 1;
						
				if ( is_array(  $this->emmisivity_correctors[ $_index ] ))
				{

					$_remainder = $a - 1 + pow( 0.1, $_index );

					$_remainder = $_remainder * pow( 10, $_index+1 );

					if ( ( ceil( $_remainder ) > 8 ) && !is_array(  $this->emmisivity_correctors[ $_index+1 ] ) )
					{
						return Model::getHighEmmissivityCorrection( $a, $calculate=true );
					}

					$_skew = $_remainder - floor( $_remainder );	


					if ( ceil( $_remainder ) > 8 )
					{
						return (1-$_skew)*$this->emmisivity_correctors[ $_index ][ floor( $_remainder )  ]['c'] + ($_skew*$this->emmisivity_correctors[ $_index+1 ][0]['c'] );
					}
					else
					{	
						return (1-$_skew)*$this->emmisivity_correctors[ $_index ][ floor( $_remainder )  ]['c'] + ($_skew*$this->emmisivity_correctors[ $_index ][ ceil( $_remainder ) ]['c'] );
					}

				}
				else
				{			
					return Model::getHighEmmissivityCorrection( $a, $calculate=true );
				}
			}
		}

		//return 4 * pow( $T, 3 ) * $deltaT * $_sum;	

		return $_sum;	
	}	

	function getSolarFlux( $n )
	{
		//this is a poor model, but it is not the end of the model we are testing, so it can be a bit mickey mouse
		//until we put together a full absorption model with proper aerosol gas profiles.
		//for Ozone absorption, it is not bad, as most of the ozone is above 100hPa
		//for H2O it is problematic. Most H2O is lower in the atmosphere, but many of the spectral absorption lines are close to saturated
		//this logarithmic profile is probably a fair wack out.

		//Despite these problems, we are only wanting a simple representation of solar absorption, and should not affect the 
		//results of testing GHG's on infrared wehich have been modelled more precisely.



		define( 'OZONE_TOTAL_ABSORPTION_RATIO', 0.08 );  //assume at O3 saturation 8% blocked
		define( 'OZONE_TOTAL_ABSORPTION_RATE', 0.045 );  // per hPa gives a roughly 99% of 8% in 100 hPa
		
		define( 'H20_TOTAL_ABSORPTION_RATIO', 0.12 );	//assume at H20 saturation 12% is blocked
		define( 'H20_TOTAL_ABSORPTION_RATE', 0.001098 );   //per hPa. gives about 66.67% of %12 absorption in 1000 hPa 
		
		define( 'SCATTERING', 0.06 );
		define( 'SURFACE_SHORTWAVE_ALBEDO', 0.08 );

		$thickness = (TOTAL_ATMOSPHERE/$this->noLayers);

		$input_flux = SOLAR_CONSTANT/4;

		//we will assume scattering is always the same, although if cloud is higher, technically scattering should be less - ah who cares - small numbers!
		$input_flux = $input_flux * ( 1 - SCATTERING );

		

		//cloud reflection
		//not worrying about absorption of reflections
		if ( $this->cloudParameters )
		{

			$input_flux = $input_flux * ( 1 - ($this->cloudParameters['cover'] * $this->cloudParameters['shortwave']['albedo']) );	

		}	
	

		$ozone_depletion_to_n = OZONE_TOTAL_ABSORPTION_RATIO * (1 - pow( (1-OZONE_TOTAL_ABSORPTION_RATE) , $thickness*($this->noLayers-$n)  ) );
		$h20_depletion_to_n = H20_TOTAL_ABSORPTION_RATIO * (1 - pow( (1-H20_TOTAL_ABSORPTION_RATE), $thickness*($this->noLayers-$n)  ));


		$ozone_depletion_to_surface = OZONE_TOTAL_ABSORPTION_RATIO * (1 - pow( (1-OZONE_TOTAL_ABSORPTION_RATE) , TOTAL_ATMOSPHERE  ) );
		$h20_depletion_to_surface = H20_TOTAL_ABSORPTION_RATIO * (1 - pow( (1-H20_TOTAL_ABSORPTION_RATE), TOTAL_ATMOSPHERE  ));


		$surface_flux = $input_flux;


		if ( $this->cloudParameters )
		{
			
			if ( !isset($this->cloudParameters['top']) || !isset($this->cloudParameters['base']) )
				die( 'Cloud Paramters top & base not set' );


			for( $k=$this->cloudParameters['top']; $k>=$this->cloudParameters['base']; $k-- )
			{
			

				if ( $n < $k )
				{
					$input_flux = $input_flux 
						* 
					( ( 1 - $this->cloudParameters['cover'] ) +  ($this->cloudParameters['cover'] * $this->cloudParameters['shortwave']['passthru'] ) );

				}

				$surface_flux = $surface_flux
					* 
				( ( 1 - $this->cloudParameters['cover'] ) +  ($this->cloudParameters['cover'] * $this->cloudParameters['shortwave']['passthru'] ) );
			}

		}		


			
		$surface_flux = (1-$ozone_depletion_to_surface-$h20_depletion_to_surface)*$surface_flux;
		$input_flux = (1-$ozone_depletion_to_n-$h20_depletion_to_n)*$input_flux;

		
		
		//the amount that reflects back and escapes through clouds

		$raw_surface_reflections =

		$surface_flux*SURFACE_SHORTWAVE_ALBEDO * 
		( 
		   ($this->cloudParameters['cover'] *  
		   pow( $this->cloudParameters['shortwave']['passthru'], $this->cloudParameters['top'] - $this->cloudParameters['base'] ))
                   +
  		   (1-$this->cloudParameters['cover'])
		);	
              
		

		//plus the infinite reflections
		$raw_surface_reflections = $raw_surface_reflections / (  1 -  ($this->cloudParameters['cover'] * $this->cloudParameters['shortwave']['albedo'] * SURFACE_SHORTWAVE_ALBEDO ) );

		//the amount of reflected light re-absorbed by the clouds

		$reabsorbed_reflections = 0;

		if ( $this->cloudParameters )
		{
			if ( $n > $this->cloudParameters['base'] )
			{
				$left_to_absorb = $surface_flux*SURFACE_SHORTWAVE_ALBEDO; 
				//and infinite reflections
				$left_to_absorb = $left_to_absorb / ( 1 -  (SURFACE_SHORTWAVE_ALBEDO * $this->cloudParameters['cover'] * $this->cloudParameters['shortwave']['albedo']) );

				for( $k=$this->cloudParameters['base']; $k<=min( ($n-1),$this->cloudParameters['top']); $k++ )
				{
		
					$reabsorbed_reflections += ($this->cloudParameters['cover'] * $left_to_absorb) * $this->cloudParameters['shortwave']['absorption'];
				
					$left_to_absorb = $left_to_absorb * (1-$this->cloudParameters['cover']); //what didnt hit cloud
					$left_to_absorb += ($this->cloudParameters['cover'] * $left_to_absorb) * $this->cloudParameters['shortwave']['passthru']; //what passed thru cloud
				
				}
			}
		}

		//echo "re-cloud absorptions: ".$reabsorbed_reflections."\n";

		//surface re-absorptions

		$surface_reabsorptions_from_cloud = (1 - SURFACE_SHORTWAVE_ALBEDO) * $surface_flux * SURFACE_SHORTWAVE_ALBEDO * $this->cloudParameters['cover'] * $this->cloudParameters['shortwave']['albedo'];

		//and the infinite reflections
		$surface_reabsorptions_from_cloud = $surface_reabsorptions_from_cloud / ( 1 -  (SURFACE_SHORTWAVE_ALBEDO * $this->cloudParameters['cover'] * $this->cloudParameters['shortwave']['albedo']) );

		//echo "surface_reabsorptions_from_cloud: ".$surface_reabsorptions_from_cloud."\n";


		return $input_flux - $raw_surface_reflections + $reabsorbed_reflections + $surface_reabsorptions_from_cloud;

	}

	function output( $_line )
	{
		if ( $this->outputFile )
		{
			if ( file_exists( $this->outputFile ) )
				$f = fopen( $this->outputFile, 'a' );		
			else
				$f = fopen( $this->outputFile, 'w' );

			
			fwrite( $f, $_line );

			fclose( $f );
		

		}
	}

	function outputStatus( $fraction, $description  )
	{
		if ( isset($fraction) )
			$this->outputFraction = $fraction;
		else
			$fraction = $this->outputFraction;

		if ( $this->outputStatusFile )
		{
			
			$f = fopen( $this->outputStatusFile, 'w' );

			
			fwrite( $f, json_encode(  array( 'fraction'=>$fraction, 'description'=>$description  )  ) );

			fclose( $f );
		}

	}

	//once layers have been calculated, the initial gas absorption data is redundant
	//to save on memory, this can be purged after create_layers.

	function purgeGasData()
	{
		unset( $this->_absorption_models );
		unset( $this->absorption_models );
		unset( $this->_decay_models );
		unset( $this->decay_models );
		unset( $this->xfade_models );
		unset( $this->_xfade_models );
		unset( $this->alpha_xfade_models );
		unset( $this->_alpha_xfade_models );

	}	

}

class Layer
{
	var $T;
	var $absSpectrum = array();

	function Layer(){}

	function createAbsorptionGraph( $output=null, $T0=NULL )
	{
		define('X_WIDTH_SPECTRUM', 750 );
		

		$_svg =  '<'.'?xml version="1.0" standalone="no"'.'?>'."\n";
		$_svg .= '<!DOCTYPE svg PUBLIC "-//W3C//DTD SVG 1.1//EN" "http://www.w3.org/Graphics/SVG/1.1/DTD/svg11.dtd">'."\n"; 
		$_svg .= '<svg width="100%" height="100%" version="1.1" xmlns="http://www.w3.org/2000/svg">'."\n";

		$_svg .=	'<line x1="10" y1="10"   x2="10"  y2="250" style="stroke: #000000; stroke-width:2;"/>'."\n";
 		$_svg .=	'<line x1="10" y1="250"  x2="'.(X_WIDTH_SPECTRUM+10).'" y2="250" style="stroke: #000000; stroke-width:2;"/>'."\n";	
		$_svg .=	'<path d="'."\n";

		$_x_scale = X_WIDTH_SPECTRUM / $this->absSpectrum[ sizeof( $this->absSpectrum )-1 ]['y'];
		$_y_scale = 240;

		$_svg .=    'M'.(($this->absSpectrum[0]['y']*$_x_scale)+10).','.( 250 - ($this->absSpectrum[0]['a']*$_y_scale))."\n";

		for( $k=1; $k<sizeof( $this->absSpectrum); $k++ )
		{
			$_svg .=    'L'.(($this->absSpectrum[$k]['y']*$_x_scale)+10).','.( 250 - ($this->absSpectrum[$k]['a']*$_y_scale))."\n";
		}
		$_svg .= '"'."\n";
		
		$_svg .= 'style="stroke: #0B0B3B; stroke-width:1; fill:none;"/>'."\n";

		if ( isset( $this->absSpectrum[0]['e'] ) )
		{
			$_svg .=	'<path d="'."\n";

			$_svg .=    'M'.(($this->absSpectrum[0]['y']*$_x_scale)+10).','.( 250 - ($this->absSpectrum[0]['e']*$_y_scale))."\n";

			for( $k=1; $k<sizeof( $this->absSpectrum); $k++ )
			{
				$_svg .=    'L'.(($this->absSpectrum[$k]['y']*$_x_scale)+10).','.( 250 - ($this->absSpectrum[$k]['e']*$_y_scale))."\n";
			}
			$_svg .= '"'."\n";
		
			$_svg .= 'style="stroke: #AAAA00; stroke-width:1; fill:none;"/>'."\n";

		}

		if( $T0 )
		{
			$maxE = Model::getEFromPlanksEquations( WIENS_CONSTANT_B * pow(10,6) / $T0, $T0 );
		}
		else
		{

			$maxE = Model::getEFromPlanksEquations( WIENS_CONSTANT_B * pow(10,6) / $this->T, $this->T );
		}


		$_svg .=	'<path d="'."\n";

		$_svg .=    'M'.(($this->absSpectrum[0]['y']*$_x_scale)+10).','.(250 - ((Model::getEFromPlanksEquations( $this->absSpectrum[0]['y'], $this->T  ) * 240) / $maxE)) ."\n";

		for( $k=1; $k<sizeof( $this->absSpectrum); $k++ )
		{
			$_svg .=    'L'.(($this->absSpectrum[$k]['y']*$_x_scale)+10).','.(250 - ((Model::getEFromPlanksEquations( $this->absSpectrum[$k]['y'], $this->T  ) * 240) / $maxE))."\n";
		}

		$_svg .= '"'."\n";
		$_svg .= 'style="stroke: #FF0000; stroke-width:1; fill:none;"/>'."\n";
		

		$_svg .= '<text x="10" y="260" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >0</text>'."\n";
		$_svg .= '<text x="'.((X_WIDTH_SPECTRUM/5)+8).'" y="265" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >20&#181;m</text>'."\n";
		$_svg .= '<line x1="'.(10 + (20*$_x_scale)).'" y1="250"   x2="'.(10 + (20*$_x_scale)).'"  y2="254" style="stroke: #000000; stroke-width:2;"/>'."\n";
		$_svg .= '<text x="'.((2*X_WIDTH_SPECTRUM/5)+6).'" y="265" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >40&#181;m</text>'."\n";
		$_svg .= '<line x1="'.(10 + (40*$_x_scale)).'" y1="250"   x2="'.(10 + (40*$_x_scale)).'"  y2="254" style="stroke: #000000; stroke-width:2;"/>'."\n";
		$_svg .= '<text x="'.((3*X_WIDTH_SPECTRUM/5)+14).'" y="265" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >60&#181;m</text>'."\n";
		$_svg .= '<line x1="'.(10 + (60*$_x_scale)).'" y1="250"   x2="'.(10 + (60*$_x_scale)).'"  y2="254" style="stroke: #000000; stroke-width:2;"/>'."\n";
		$_svg .= '<text x="'.((4*X_WIDTH_SPECTRUM/5)+2).'" y="265" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >80&#181;m</text>'."\n";
		$_svg .= '<line x1="'.(10 + (80*$_x_scale)).'" y1="250"   x2="'.(10 + (80*$_x_scale)).'"  y2="254" style="stroke: #000000; stroke-width:2;"/>'."\n";
		$_svg .= '<text x="'.( X_WIDTH_SPECTRUM ).'" y="265" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >100&#181;m</text>'."\n";
		$_svg .= '<line x1="'.(10 + (100*$_x_scale)).'" y1="250"   x2="'.(10 + (100*$_x_scale)).'"  y2="254" style="stroke: #000000; stroke-width:2;"/>'."\n";


		if ( isset( $this->absSpectrum[0]['e'] ) )
		{
			$_svg .= '<line x1="10" y1="270"   x2="40"  y2="270" style="stroke: #0B0B3B; stroke-width:2;"/>'."\n";
			$_svg .= '<text x="50" y="274" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >Not Transmitted</text>'."\n";

			$_svg .= '<line x1="10" y1="290"   x2="40"  y2="290" style="stroke: #AAAA00; stroke-width:2;"/>'."\n";
			$_svg .= '<text x="50" y="294" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >Emmisivity (Cloud Affected)</text>'."\n";

			$_svg .= '<line x1="10" y1="310"   x2="40"  y2="310" style="stroke: #FF0000; stroke-width:2;"/>'."\n";
			$_svg .= '<text x="50" y="314" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >Black Body Curve ('.round($this->T,1).'&#176;)</text>'."\n";

		}
		else
		{
			$_svg .= '<line x1="10" y1="270"   x2="40"  y2="270" style="stroke: #0B0B3B; stroke-width:2;"/>'."\n";
			$_svg .= '<text x="50" y="274" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >Absorbed</text>'."\n";

			$_svg .= '<line x1="10" y1="290"   x2="40"  y2="290" style="stroke: #FF0000; stroke-width:2;"/>'."\n";
			$_svg .= '<text x="50" y="294" style="stroke:#000000; fill: #000000; font-family:Arial; font-size: 10px;" >Black Body Curve ('.round($this->T,1).'&#176;)</text>'."\n";
			
		}

		$_svg .= '</svg>';

		if ( $output )
		{
			file_put_contents( $output, $_svg  );
		}

		return $_svg;
	}

}


?>
