<?
ini_set('memory_limit','800M');
set_time_limit(0);

include( 'climateModel.class.php' );

define( 'USER_SPACE', 'db' );
define( 'USER_FILE', 'user.data' );
define( 'TOLERANCE', 0.05 );


$config['ghg']['wet']['H20']['description'] = 'H2O - Water Vapour';
$config['ghg']['wet']['H20']['referenceT'] = 293;	//degrees Kelvin
$config['ghg']['wet']['H20']['referenceH'] = 50;	//%	
$config['ghg']['wet']['H20']['parameters'] = 
						array(  'alpha'=>'absorption_gasses/H2O-950hPa-combined.csv', 
						'decay'=>'absorption_gasses/H2O-spectra.decay', 
						'xfade'=>'absorption_gasses/H2O-spectra.xfade', 
						'alpha_xfade'=> 'absorption_gasses/H2O-50hPa-combined.csv', 
						'thickness'=> 1,
						'scale'=>1, 
						'scenario'=>1  );


$config['ghg']['dry']['CO2']['description'] = 'CO2 - Carbon Dioxide';
$config['ghg']['dry']['CO2']['concentration'] = 388;
$config['ghg']['dry']['CO2']['units'] = 'PPM';
$config['ghg']['dry']['CO2']['parameters'] = 
					array(   'alpha'=>'absorption_gasses/CO2-950hPa-combined.csv', 
					'decay'=>'absorption_gasses/CO2-spectra.decay', 
					'xfade'=>'absorption_gasses/CO2-spectra.xfade' , 
					'alpha_xfade'=>'absorption_gasses/CO2-50hPa-combined.csv', 
					'thickness'=> 10,
					'scale'=>1, 
					'scenario'=>1 );

$config['ghg']['dry']['CH4']['description'] = 'CH4 - Methane';
$config['ghg']['dry']['CH4']['concentration'] = 1.775;
$config['ghg']['dry']['CH4']['units'] = 'PPM';
$config['ghg']['dry']['CH4']['parameters'] = 
					array(   'alpha'=>'absorption_gasses/CH4-950hPa-combined.csv', 
					'decay'=>'absorption_gasses/CH4-spectra.decay', 
					'xfade'=>'absorption_gasses/CH4-spectra.xfade' , 
					'alpha_xfade'=>'absorption_gasses/CH4-50hPa-combined.csv', 
					'thickness'=> 10,
					'scale'=>1, 
					'scenario'=>1 );

$config['ghg']['dry']['N2O']['description'] = 'N2O - Nitrous Oxide';
$config['ghg']['dry']['N2O']['concentration'] = 314;
$config['ghg']['dry']['N2O']['units'] = 'PPB';
$config['ghg']['dry']['N2O']['parameters'] = 
					array(   'alpha'=>'absorption_gasses/N2O-950hPa-combined.csv', 
					'decay'=>'absorption_gasses/N2O-spectra.decay', 
					'xfade'=>'absorption_gasses/N2O-spectra.xfade' , 
					'alpha_xfade'=>'absorption_gasses/N2O-50hPa-combined.csv', 
					'thickness'=> 10,
					'scale'=>1, 
					'scenario'=>1 );

$config['ghg']['dry']['O3']['description'] = 'O3 - Ozone';
$config['ghg']['dry']['O3']['concentration'] = 5;
$config['ghg']['dry']['O3']['units'] = 'PPB';
$config['ghg']['dry']['O3']['parameters'] = 
					array(   'alpha'=>'absorption_gasses/O3-950hPa-combined.csv', 
					'decay'=>'absorption_gasses/O3-spectra.decay', 
					'xfade'=>'absorption_gasses/O3-spectra.xfade' , 
					'alpha_xfade'=>'absorption_gasses/O3-50hPa-combined.csv', 
					'thickness'=> 10,
					'scale'=>1, 
					'scenario'=>1 );

$config['atmosphere']['default'] = array(
					'relhumidity'=>array( 50,50,50,50,50,50,45,40,30,20 ),
					'vapour_feedback'=>true,
					'cloudLayer'=>5,
					'cloudCover'=>35,
					'cloudShortWave'=>array('albedo'=>55,'absorption'=>35,'passthru'=>10),
					'cloudLongWave'=>array('albedo'=>55,'absorption'=>35,'passthru'=>10  ),
					'dryGas'=>array( 'CO2'=>array( 'scenario'=>1 ), 'CH4'=>array( 'scenario'=>1 ),'N2O'=>array( 'scenario'=>1 ),'O3'=>array( 'scenario'=>1 ) ),
 					'add_dry_convection'=>1,
					'add_wet_convection'=>1,
                                        'coefficient_of_dry_convection'=>COEFFICIENT_OF_DRY_CONVECTION,
				   	'coefficient_of_wet_convection'=>COEFFICIENT_OF_WET_CONVECTION
				);

$config['colours'] = array( '#000000', '#B40404','#DBA901','#01DF01','#00FFFF','#0040FF','#BF00FF','#FE2E64','#DF7401','#088A08','#0B3B39','#3B0B0B','#0B173B','#240B3B','#3B0B2E' );		

function getColour( $n )
{
	global $config;

	if ( $n>=sizeof( $config['colours'] ) );
		$n = $n % sizeof( $config['colours'] );

	return $config['colours'][ $n ];
}


function saveStatus( $Model = null, $_run = NULL )
{
	global $Model, $_user_status, $user_id, $bool_read_only, $_SESSION;

	if ( $bool_read_only ){

		session_start();
		$_SESSION['_user_status'] = $_user_status;
		session_write_close();
		return;
	}

	if ( !is_dir( '../'.USER_SPACE.'/'.$user_id ) )
	{
		mkdir( '../'.USER_SPACE.'/'.$user_id, 0777 );

	}

	file_put_contents( '../'.USER_SPACE.'/'.$user_id.'/'.USER_FILE, serialize( $_user_status )  );

	if ( $Model )
	{
		if ( !$_run )
			$_run = $_user_status['current_run'];

		if ( !is_dir( '../'.USER_SPACE.'/'.$user_id.'/run_'.$_run ) )
		{	
			mkdir( '../'.USER_SPACE.'/'.$user_id.'/run_'.$_run, 0777 );
		}

		file_put_contents( '../'.USER_SPACE.'/'.$user_id.'/run_'.$_run.'/model.data' , serialize( $Model )  );
	}	
}

function getCurrentModel( $_run=NULL )
{
	global $Model, $_user_status, $user_id;

	if( !$_run )
		$_run =  $_user_status['current_run'];



	if ( file_exists( '../'.USER_SPACE.'/'.$user_id.'/run_'.$_run.'/model.data' ) )
	{
		$_dt = file_get_contents( '../'.USER_SPACE.'/'.$user_id.'/run_'.$_run.'/model.data' );
		
		

		return unserialize( $_dt );
	}
	
	return false;
}

function deleteDirectory($dir) {
    if (!file_exists($dir)) return true;
    if (!is_dir($dir) || is_link($dir)) return unlink($dir);
        foreach (scandir($dir) as $item) {
            if ($item == '.' || $item == '..') continue;
            if (!deleteDirectory($dir . "/" . $item)) {
                chmod($dir . "/" . $item, 0777);
                if (!deleteDirectory($dir . "/" . $item)) return false;
            };
        }
        return rmdir($dir);
    } 


//var_dump( $_COOKIE );
//exit;

if ( $user_id = $_COOKIE['user_id'] )
{
	@setcookie( 'user_id', $user_id, mktime() + 30*24*60*60, '/' );
}



if( file_exists( '../'.USER_SPACE.'/'.$user_id.'/'.USER_FILE ) )
{
	if( !is_writable( '../'.USER_SPACE.'/'.$user_id ) )
	{

		//for readonly projects, store status as session
		$bool_read_only = true;

		session_start();

		if ( $_SESSION['_user_status'] &&  ($_SESSION['user_id'] == $user_id) ){
			$_user_status = $_SESSION['_user_status'];
		}else{
			$_dt = file_get_contents( '../'.USER_SPACE.'/'.$user_id.'/'.USER_FILE );
			$_user_status = unserialize( $_dt );

			$_SESSION['_user_status'] = $_user_status;
			$_SESSION['user_id'] = $user_id;

		}

		session_write_close();
	}
	else
	{
		$_dt = file_get_contents( '../'.USER_SPACE.'/'.$user_id.'/'.USER_FILE );
		$_user_status = unserialize( $_dt );

	}
		
}
else
	$_user_status = array();



?>
