#!/usr/bin/php
<?
/*
//This is a command line version of the climate model
//You will require a CLI version of php to run
//
*/

define( 'TOLERANCE', 0.05 );

ini_set('memory_limit','800M');
set_time_limit(0);

chdir( 'include' );

include( 'climateModel.class.php' );


$_model = new Model();
$_model->verbose = true;


$_gasses = array( 
		array(  'alpha'=>'absorption_gasses/H2O-950hPa-combined.csv', //the primary lower atmosphere spectrum file
			'decay'=>'absorption_gasses/H2O-spectra.decay',      //the file that defines how the concentrations change with height	
			'xfade'=>'absorption_gasses/H2O-spectra.xfade',      //the file that defines the xface parameters with the upper atmosphere spectrum
			'alpha_xfade'=> 'absorption_gasses/H2O-50hPa-combined.csv',  //the upper atmosphere spectrum
			'thickness'=> 1,						//the thickness in hPa of atrmosphere that the spectrum represents	
			'scale'=>1, 							//the scale of the absorption data, 1 means 0 - 1, 100 would be for percentages
			'scenario'=>1  ),						//the number to change for multiplying the concentration, to simulate different scenarios
		
		array(	'alpha'=>'absorption_gasses/CO2-950hPa-combined.csv', 
			'decay'=>'absorption_gasses/CO2-spectra.decay', 
			'xfade'=>'absorption_gasses/CO2-spectra.xfade' , 
			'alpha_xfade'=>'absorption_gasses/CO2-50hPa-combined.csv', 
			'thickness'=> 10,
			'scale'=>1, 
			'scenario'=>1 ),
		array(  'alpha'=>'absorption_gasses/CH4-950hPa-combined.csv', 
			'decay'=>'absorption_gasses/CH4-spectra.decay', 
			'xfade'=>'absorption_gasses/CH4-spectra.xfade' , 
			'alpha_xfade'=>'absorption_gasses/CH4-50hPa-combined.csv', 
			'thickness'=> 10,
			'scale'=>1, 
			'scenario'=>1 ),
		array(  'alpha'=>'absorption_gasses/N2O-950hPa-combined.csv', 
			'decay'=>'absorption_gasses/N2O-spectra.decay', 
			'xfade'=>'absorption_gasses/N2O-spectra.xfade' , 
			'alpha_xfade'=>'absorption_gasses/N2O-50hPa-combined.csv', 
			'thickness'=> 10,
			'scale'=>1, 
			'scenario'=>1 ),
		array(  'alpha'=>'absorption_gasses/O3-950hPa-combined.csv', 
			'decay'=>'absorption_gasses/O3-spectra.decay', 
			'xfade'=>'absorption_gasses/O3-spectra.xfade' , 
			'alpha_xfade'=>'absorption_gasses/O3-50hPa-combined.csv', 
			'thickness'=> 10,
			'scale'=>1, 
			'scenario'=>1 )   
		);

$clouds=array(  'base'=>5,			//the base level that has cloud
		'top'=>5, 			//the top level that has cloud (ie cloud can span levels if so desired)
		'cover'=>0.35, 			//fraction of sky covered by cloud
		'shortwave'=>array(
				'albedo'=>0.55,		//solar reflection
				'absorption'=>0.35,	//solar absorption
				'passthru'=>0.10, ),	//solar light that gets through

		'longwave'=>array(
				'albedo'=>0.55,		//infrared relection
				'absorption'=>0.35,	//infrared absorption
				'passthru'=>0.10  ) 	//infrared that gets through
		); 

$_model->loadAbsorptionSpectrums( $_gasses );

$_model->setClouds( $clouds );
$_model->createLayers( NO_LAYERS );
$_model->purgeGasData();

while( true )
{		
	$_model->create_Mattrix();
	$_model->solveMattrix();

	if ( $_model->getDeviation() < TOLERANCE )
		break;

	$_model->bounceCorrection();	//adjusts the starting temperatures if the convergence is bouncing
}
?>
